@@ -72,6 +72,7 @@ namespace KBibTeX
         void copyReferences();
         void cut();
         bool paste();
+        bool paste( const QString& text, DocumentListViewItem *at );
         void selectAll();
         void filter( const QString& text, BibTeX::Element::FilterType filterType, BibTeX::EntryField::FieldType fieldType );
         void setReadOnly( bool isReadOnly );
@@ -93,11 +94,7 @@ namespace KBibTeX
         void makeNewItemsUnread();
 
     protected:
-        //     void resizeEvent( QResizeEvent * e );
-        //     void contentsDragEnterEvent( QDragEnterEvent *e );
-        //         void dropEvent( QDropEvent *e );
         bool acceptDrag( QDropEvent* e ) const;
-        //     void contentsDropEvent( QDropEvent *e );
         void startDrag();
 
         virtual bool eventFilter( QObject *watched, QEvent *e );
@@ -120,7 +117,6 @@ namespace KBibTeX
         int m_newElementCounter;
 
         void setItems();
-        //         void appendBibTeXFile( BibTeX::File *bibtexfile );
         QString selectedToBibTeXText();
         QString selectedToBibTeXRefs();
 
