@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2008 by Thomas Fischer                             *
+*   Copyright (C) 2004-2009 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -35,6 +35,9 @@
 #include <kdebug.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
+#include <kapplication.h>
+#include <kconfig.h>
+#include <kwin.h>
 
 #include <entry.h>
 #include <entryfield.h>
@@ -49,7 +52,6 @@
 #include <entrywidgetexternal.h>
 #include <entrywidgetuserdefined.h>
 #include <entrywidgetother.h>
-#include <entrywidgetsource.h>
 #include <entrywidgetwarningsitem.h>
 #include <fieldlistview.h>
 #include <idsuggestions.h>
@@ -71,8 +73,8 @@ namespace KBibTeX
         return result;
     }
 
-    EntryWidget::EntryWidget( BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QWidget *parent, const char *name )
-            : QWidget( parent, name ), m_originalEntry( NULL ), m_bibtexfile( bibtexfile ), m_isReadOnly( isReadOnly ), m_isNew( isNew ), m_lastPage( NULL )
+    EntryWidget::EntryWidget( BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QDialog *parent, const char *name )
+            : QWidget( parent, name ), m_originalEntry( NULL ), m_bibtexfile( bibtexfile ), m_isReadOnly( isReadOnly ), m_isNew( isNew ), m_lastPage( NULL ), m_dlgParent( parent )
     {
         setupGUI( parent, FALSE );
 
@@ -83,8 +85,8 @@ namespace KBibTeX
         m_pushButtonIdSuggestions->setEnabled( !m_isReadOnly );
     }
 
-    EntryWidget::EntryWidget( BibTeX::Entry *entry, BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QWidget *parent, const char *name )
-            : QWidget( parent, name ), m_originalEntry( entry ), m_bibtexfile( bibtexfile ), m_isReadOnly( isReadOnly ), m_isNew( isNew ), m_lastPage( NULL )
+    EntryWidget::EntryWidget( BibTeX::Entry *entry, BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QDialog *parent, const char *name )
+            : QWidget( parent, name ), m_originalEntry( entry ), m_bibtexfile( bibtexfile ), m_isReadOnly( isReadOnly ), m_isNew( isNew ), m_lastPage( NULL ), m_dlgParent( parent )
     {
         setupGUI( parent );
 
@@ -101,6 +103,10 @@ namespace KBibTeX
     {
         m_updateWarningsTimer->stop();
         delete m_updateWarningsTimer;
+
+        KConfig * config = kapp->config();
+        config->setGroup( "EntryWidget" );
+        saveWindowSize( config );
     }
 
     void EntryWidget::showEvent( QShowEvent *showev )
@@ -109,9 +115,13 @@ namespace KBibTeX
         EntryWidgetTitle *ewt = dynamic_cast<EntryWidgetTitle*>( m_tabWidget->page( 0 ) );
         if ( ewt != NULL )
             ewt->m_fieldLineEditTitle->setFocus();
+
+        KConfig * config = kapp->config();
+        config->setGroup( "EntryWidget" );
+        restoreWindowSize( config );
     }
 
-    void EntryWidget::closeEvent( QCloseEvent * e )
+    bool EntryWidget::queryClose()
     {
         bool isModified = m_lineEditID->isModified();
 
@@ -120,16 +130,19 @@ namespace KBibTeX
         isModified |= m_sourcePage->isModified();
 
         KGuiItem discardBtn = KGuiItem( i18n( "Discard" ), "editshred" );
-        if ( isModified && KMessageBox::warningContinueCancel( this, i18n( "The current entry has been modified. Do you want do discard your changes?" ), i18n( "Discard changes" ), discardBtn ) == KMessageBox::Cancel )
-            e->ignore();
-        else
-            e->accept();
+        return !isModified || KMessageBox::warningContinueCancel( this, i18n( "The current entry has been modified. Do you want do discard your changes?" ), i18n( "Discard changes" ), discardBtn ) == KMessageBox::Continue;
     }
 
     void EntryWidget::apply()
     {
         if ( !m_isReadOnly )
+        {
             apply( m_originalEntry );
+            m_originalEntry->setId( IdSuggestions::resolveConflict( m_bibtexfile, m_originalEntry->id(), m_originalEntry ) );
+
+            Settings * settings = Settings::self();
+            settings->addToCompletion( m_originalEntry );
+        }
     }
 
     void EntryWidget::apply( BibTeX::Entry *entry )
@@ -141,9 +154,6 @@ namespace KBibTeX
         {
             for ( QValueList<KBibTeX::EntryWidgetTab*>::iterator it( m_internalEntryWidgets.begin() ); it != m_internalEntryWidgets.end(); ++it )
                 ( *it ) ->apply( entry );
-
-            Settings * settings = Settings::self();
-            settings->addToCompletion( entry );
         }
     }
 
@@ -155,8 +165,7 @@ namespace KBibTeX
         else
             entry->setEntryType( entryType );
 
-        QString id = m_lineEditID->text();
-        entry->setId( id );
+        entry->setId( m_lineEditID->text() );
     }
 
     void EntryWidget::reset()
@@ -231,18 +240,40 @@ namespace KBibTeX
         else if ( m_lastPage == m_sourcePage )
         {
             // switching from source tab away
-            m_sourcePage->apply( &temporaryEntry );
-            internalReset( &temporaryEntry );
-            for ( QValueList<KBibTeX::EntryWidgetTab*>::iterator it( m_internalEntryWidgets.begin() ); it != m_internalEntryWidgets.end(); ++it )
-                ( *it ) ->reset( &temporaryEntry );
-            updateWarnings();
+            bool doApply = true;
+            bool doChange = true;
+            if ( !m_sourcePage->containsValidText() )
+            {
+                if ( KMessageBox::warningYesNo( this, i18n( "The source code does not contain valid BibTeX code.\n\nRestore previous version or continue editing?" ), i18n( "Invalid BibTeX code" ), KGuiItem( i18n( "Restore" ) ), KGuiItem( i18n( "Edit" ) ) ) == KMessageBox::No )
+                {
+                    QString text = m_sourcePage->text();
+                    m_tabWidget->showPage( m_sourcePage );
+                    m_sourcePage->setText( text );
+                    doChange = false;
+                    newPage = m_sourcePage;
+                }
+                else
+                    doApply = false;
+            }
 
-            m_comboBoxEntryType->setEnabled( TRUE );
-            m_lineEditID->setEnabled( !m_defaultIdSuggestionAvailable || !m_pushButtonForceDefaultIdSuggestion->isOn() );
-            m_pushButtonIdSuggestions->setEnabled(( !m_defaultIdSuggestionAvailable || !m_pushButtonForceDefaultIdSuggestion->isOn() ) && !m_isReadOnly );
-            m_pushButtonForceDefaultIdSuggestion->setEnabled( !m_isReadOnly && m_defaultIdSuggestionAvailable );
-            if ( !m_isReadOnly )
-                m_updateWarningsTimer->start( 500 );
+            if ( doChange )
+            {
+                if ( doApply )
+                {
+                    m_sourcePage->apply( &temporaryEntry );
+                    internalReset( &temporaryEntry );
+                    for ( QValueList<KBibTeX::EntryWidgetTab*>::iterator it( m_internalEntryWidgets.begin() ); it != m_internalEntryWidgets.end(); ++it )
+                        ( *it ) ->reset( &temporaryEntry );
+                    updateWarnings();
+                }
+
+                m_comboBoxEntryType->setEnabled( TRUE );
+                m_lineEditID->setEnabled( !m_defaultIdSuggestionAvailable || !m_pushButtonForceDefaultIdSuggestion->isOn() );
+                m_pushButtonIdSuggestions->setEnabled(( !m_defaultIdSuggestionAvailable || !m_pushButtonForceDefaultIdSuggestion->isOn() ) && !m_isReadOnly );
+                m_pushButtonForceDefaultIdSuggestion->setEnabled( !m_isReadOnly && m_defaultIdSuggestionAvailable );
+                if ( !m_isReadOnly )
+                    m_updateWarningsTimer->start( 500 );
+            }
         }
 
         m_lastPage = newPage;
@@ -268,7 +299,7 @@ namespace KBibTeX
 
     void EntryWidget::setupGUI( QWidget *parent, bool showWarnings )
     {
-        QGridLayout * layout = new QGridLayout( this, 4, 6, 0, KDialog::marginHint() );
+        QGridLayout * layout = new QGridLayout( this, 4, 6, 0, KDialog::spacingHint() );
 
         // in the top row on the left, put an entry type label and combobox
         QLabel *label = new QLabel( i18n( "E&ntry Type:" ), this );
@@ -394,18 +425,13 @@ namespace KBibTeX
                 new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlWarning, i18n( "Please supply more fields to use the default id" ), m_pushButtonForceDefaultIdSuggestion, m_listViewWarnings );
             else
             {
-                QString conflictFreeId = id;
-                int nr = 0;
-                while ( m_bibtexfile->containsKey( conflictFreeId ) != NULL )
-                    conflictFreeId = QString( "%1-%2" ).arg( id ).arg( ++nr );
-
-                new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlInformation, QString( i18n( "Using '%1' as entry id" ) ).arg( conflictFreeId ), m_pushButtonForceDefaultIdSuggestion, m_listViewWarnings );
-                m_lineEditID->setText( conflictFreeId );
+                new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlInformation, QString( i18n( "Using '%1' as entry id" ) ).arg( id ), m_pushButtonForceDefaultIdSuggestion, m_listViewWarnings );
+                m_lineEditID->setText( id );
             }
         }
         else if ( m_lineEditID->text().isEmpty() )
             new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlError, i18n( "An entry has to have an identifier" ), m_lineEditID, m_listViewWarnings );
-        else if (( p = m_lineEditID->text().find( QRegExp( "[^-.:/+_a-zA-Z0-9]" ) ) ) > 0 )
+        else if (( p = m_lineEditID->text().find( Settings::noIdChars ) ) > 0 )
             new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlError, QString( i18n( "The identifier contains invalid characters at position %1" ) ).arg( p + 1 ), m_lineEditID, m_listViewWarnings );
 
         BibTeX::Entry::EntryType entryType = BibTeX::Entry::entryTypeFromString( m_comboBoxEntryType->currentText() );
@@ -426,10 +452,19 @@ namespace KBibTeX
             ( *it ) ->apply( &temporaryEntry );
         internalApply( &temporaryEntry );
 
+        /** check if entry with same id already exists */
+        QString id = temporaryEntry.id();
+        if ( m_bibtexfile != NULL && !id.isEmpty() )
+        {
+            BibTeX::Entry *nameMatched = dynamic_cast<BibTeX::Entry*>( m_bibtexfile->containsKey( id ) );
+            if ( nameMatched != NULL && nameMatched != m_originalEntry )
+                new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlError, QString( i18n( "This BibTeX file already contains an entry with id '%1'." ) ).arg( id ), m_lineEditID, m_listViewWarnings );
+        }
+
         BibTeX::Entry *crossRefEntry = NULL;
         QString crossRefText = "";
         BibTeX::EntryField *crossRef = temporaryEntry.getField( BibTeX::EntryField::ftCrossRef );
-        if ( crossRef != NULL )
+        if ( crossRef != NULL && m_bibtexfile != NULL )
         {
             crossRefText = crossRef->value()->text();
             crossRefEntry = dynamic_cast<BibTeX::Entry*>( m_bibtexfile->containsKey( crossRefText ) );
@@ -444,33 +479,33 @@ namespace KBibTeX
             }
             break;
         case BibTeX::Entry::etInProceedings:
+        {
+            if ( crossRefEntry != NULL )
             {
-                if ( crossRefEntry != NULL )
+                if ( crossRefEntry->getField( BibTeX::EntryField::ftEditor ) == NULL && crossRefEntry->getField( BibTeX::EntryField::ftKey ) == NULL && crossRefEntry->getField( BibTeX::EntryField::ftBookTitle ) == NULL )
                 {
-                    if ( crossRefEntry->getField( BibTeX::EntryField::ftEditor ) == NULL && crossRefEntry->getField( BibTeX::EntryField::ftKey ) == NULL && crossRefEntry->getField( BibTeX::EntryField::ftBookTitle ) == NULL )
-                    {
-                        new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlWarning, QString( i18n( "Cross referenced entry '%1' must contain either 'Editor', 'Key', or 'Book Title'" ) ).arg( crossRefText ), NULL, m_listViewWarnings );
-                    }
+                    new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlWarning, QString( i18n( "Cross referenced entry '%1' must contain either 'Editor', 'Key', or 'Book Title'" ) ).arg( crossRefText ), NULL, m_listViewWarnings );
                 }
             }
-            break;
+        }
+        break;
         case BibTeX::Entry::etInBook:
+        {
+            if ( crossRefEntry != NULL )
             {
-                if ( crossRefEntry != NULL )
+                if ( crossRefEntry->getField( BibTeX::EntryField::ftVolume ) == NULL )
+                    new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlWarning, QString( i18n( "Cross referenced entry '%1' must contain 'Volume'" ) ).arg( crossRefText ), NULL, m_listViewWarnings );
+                if ( crossRefEntry->getField( BibTeX::EntryField::ftEditor ) == NULL && crossRefEntry->getField( BibTeX::EntryField::ftKey ) == NULL && crossRefEntry->getField( BibTeX::EntryField::ftSeries ) == NULL )
                 {
-                    if ( crossRefEntry->getField( BibTeX::EntryField::ftVolume ) == NULL )
-                        new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlWarning, QString( i18n( "Cross referenced entry '%1' must contain 'Volume'" ) ).arg( crossRefText ), NULL, m_listViewWarnings );
-                    if ( crossRefEntry->getField( BibTeX::EntryField::ftEditor ) == NULL && crossRefEntry->getField( BibTeX::EntryField::ftKey ) == NULL && crossRefEntry->getField( BibTeX::EntryField::ftSeries ) == NULL )
-                    {
-                        new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlWarning, QString( i18n( "Cross referenced entry '%1' must contain either 'Editor', 'Key', or 'Series'" ) ).arg( crossRefText ), NULL, m_listViewWarnings );
-                    }
+                    new KBibTeX::EntryWidgetWarningsItem( KBibTeX::EntryWidgetWarningsItem::wlWarning, QString( i18n( "Cross referenced entry '%1' must contain either 'Editor', 'Key', or 'Series'" ) ).arg( crossRefText ), NULL, m_listViewWarnings );
                 }
             }
-            break;
+        }
+        break;
         default:
-            {
+        {
 // nothing
-            }
+        }
         }
     }
 
@@ -493,5 +528,85 @@ namespace KBibTeX
     {
         m_lineEditID->setText( m_idToSuggestion[id] );
     }
+
+    /* This function was taken form KMainWindow of KDE 3.5 and modified to fit KBibTeX */
+    void EntryWidget::saveWindowSize( KConfig *config ) const
+    {
+        int scnum = QApplication::desktop()->screenNumber( parentWidget() );
+        QRect desk = QApplication::desktop()->screenGeometry( scnum );
+        int w, h;
+#if defined Q_WS_X11
+        // save maximalization as desktop size + 1 in that direction
+        KWin::WindowInfo info = KWin::windowInfo( m_dlgParent->winId(), NET::WMState );
+        w = info.state() & NET::MaxHoriz ? desk.width() + 1 : m_dlgParent->width();
+        h = info.state() & NET::MaxVert ? desk.height() + 1 : m_dlgParent->height();
+#else
+        if ( isMaximized() )
+        {
+            w = desk.width() + 1;
+            h = desk.height() + 1;
+        }
+        //TODO: add "Maximized" property instead "+1" hack
+#endif
+        QRect size( desk.width(), w, desk.height(), h );
+        bool defaultSize = false;//( size == d->defaultWindowSize );
+        QString widthString = QString::fromLatin1( "Width %1" ).arg( desk.width() );
+        QString heightString = QString::fromLatin1( "Height %1" ).arg( desk.height() );
+        if ( !config->hasDefault( widthString ) && defaultSize )
+            config->revertToDefault( widthString );
+        else
+            config->writeEntry( widthString, w );
+
+        if ( !config->hasDefault( heightString ) && defaultSize )
+            config->revertToDefault( heightString );
+        else
+            config->writeEntry( heightString, h );
+    }
+
+    /* This function was taken form KMainWindow of KDE 3.5 and modified to fit KBibTeX */
+    void EntryWidget::restoreWindowSize( KConfig *config )
+    {
+        // restore the size
+        int scnum = QApplication::desktop()->screenNumber( parentWidget() );
+        QRect desk = QApplication::desktop()->screenGeometry( scnum );
+//         if ( d->defaultWindowSize.isNull() ) // only once
+//             d->defaultWindowSize = QRect( desk.width(), width(), desk.height(), height() ); // store default values
+        QSize size( config->readNumEntry( QString::fromLatin1( "Width %1" ).arg( desk.width() ), 0 ),
+                    config->readNumEntry( QString::fromLatin1( "Height %1" ).arg( desk.height() ), 0 ) );
+        if ( size.isEmpty() )
+        {
+            // try the KDE 2.0 way
+            size = QSize( config->readNumEntry( QString::fromLatin1( "Width" ), 0 ),
+                          config->readNumEntry( QString::fromLatin1( "Height" ), 0 ) );
+            if ( !size.isEmpty() )
+            {
+                // make sure the other resolutions don't get old settings
+                config->writeEntry( QString::fromLatin1( "Width" ), 0 );
+                config->writeEntry( QString::fromLatin1( "Height" ), 0 );
+            }
+        }
+        if ( !size.isEmpty() )
+        {
+#ifdef Q_WS_X11
+            int state = ( size.width() > desk.width() ? NET::MaxHoriz : 0 )
+                        | ( size.height() > desk.height() ? NET::MaxVert : 0 );
+            if (( state & NET::Max ) == NET::Max )
+                ; // no resize
+            else if (( state & NET::MaxHoriz ) == NET::MaxHoriz )
+                m_dlgParent->resize( width(), size.height() );
+            else if (( state & NET::MaxVert ) == NET::MaxVert )
+                m_dlgParent->resize( size.width(), height() );
+            else
+                m_dlgParent->resize( size );
+            // QWidget::showMaximized() is both insufficient and broken
+            KWin::setState( m_dlgParent->winId(), state );
+#else
+            if ( size.width() > desk.width() || size.height() > desk.height() )
+                m_dlgParent->setWindowState( WindowMaximized );
+            else
+                m_dlgParent->resize( size );
+#endif
+        }
+    }
 }
 #include "entrywidget.moc"
