@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2008 by Thomas Fischer                             *
+*   Copyright (C) 2004-2009 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -259,7 +259,10 @@ namespace KBibTeX
 
         addMissingWarning( entryType, BibTeX::EntryField::ftHowPublished, m_fieldLineEditHowPublished->caption(), !m_fieldLineEditHowPublished->isEmpty(), m_fieldLineEditHowPublished, listViewWarnings );
 
-        addMissingWarning( entryType, BibTeX::EntryField::ftPublisher, m_fieldLineEditPublisher->caption(), !m_fieldLineEditPublisher->isEmpty(), m_fieldLineEditPublisher, listViewWarnings );
+        crossRefValid = m_crossRefEntry != NULL && m_crossRefEntry->getField( BibTeX::EntryField::ftPublisher ) != NULL;
+        addMissingWarning( entryType, BibTeX::EntryField::ftPublisher, m_fieldLineEditPublisher->caption(), !m_fieldLineEditPublisher->isEmpty() || crossRefValid, m_fieldLineEditPublisher, listViewWarnings );
+        if ( crossRefValid && m_fieldLineEditPublisher->isEmpty() )
+            addCrossRefInfo( m_fieldLineEditPublisher->caption(), m_fieldLineEditPublisher, listViewWarnings );
 
         addMissingWarning( entryType, BibTeX::EntryField::ftOrganization, m_fieldLineEditOrganization->caption(), !m_fieldLineEditOrganization->isEmpty(), m_fieldLineEditOrganization, listViewWarnings );
 
@@ -375,7 +378,7 @@ namespace KBibTeX
     void EntryWidgetPublication::slotSetCrossRefEntry()
     {
         m_crossRefEntry = NULL;
-        if ( m_fieldLineEditCrossRef->value() != NULL )
+        if ( m_fieldLineEditCrossRef->value() != NULL && m_bibtexfile != NULL )
         {
             QString crossRefText = m_fieldLineEditCrossRef->value() ->text();
             if ( !crossRefText.isEmpty() )
