@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2008 by Thomas Fischer                             *
+*   Copyright (C) 2004-2009 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -32,22 +32,20 @@ namespace BibTeX
 {
 
     FileExporterBibTeX::FileExporterBibTeX() : FileExporter(),
-            m_stringOpenDelimiter( '"' ), m_stringCloseDelimiter( '"' ), m_encoding( File::encLaTeX ), m_protectCasing( FALSE ), cancelFlag( FALSE )
+            m_iconvBufferSize( 16384 ), m_stringOpenDelimiter( '"' ), m_stringCloseDelimiter( '"' ), m_keywordCasing( kcCamelCase ), m_encoding( "latex" ), m_protectCasing( FALSE ), cancelFlag( FALSE )
     {
-        // nothing
+        m_iconvBuffer = new char[m_iconvBufferSize];
     }
 
     FileExporterBibTeX::~FileExporterBibTeX()
     {
-        // nothing
+        delete[] m_iconvBuffer;
     }
 
     bool FileExporterBibTeX::save( QIODevice* iodevice, const File* bibtexfile, QStringList * /*errorLog*/ )
     {
+        m_mutex.lock();
         bool result = TRUE;
-        QTextStream stream( iodevice );
-        if ( m_encoding == File::encUTF8 )
-            stream.setEncoding( QTextStream::UnicodeUTF8 );
 
         /**
           * Categorize elements from the bib file into four groups,
@@ -55,6 +53,7 @@ namespace BibTeX
           * in the correct order.
           */
 
+        QValueList<Comment*> parameterCommentsList;
         QValueList<Preamble*> preambleList;
         QValueList<Macro*> macroList;
         QValueList<Entry*> crossRefingEntryList;
@@ -76,68 +75,108 @@ namespace BibTeX
                     if (( entry != NULL ) && ( entry->getField( EntryField::ftCrossRef ) != NULL ) )
                         crossRefingEntryList.append( entry );
                     else
-                        remainingList.append( *it );
+                    {
+                        Comment *comment = dynamic_cast<Comment*>( *it );
+                        QString commentText = QString::null;
+                        /** check if this file requests a special encoding */
+                        if ( comment != NULL && comment->useCommand() && (( commentText = comment->text().lower() ) ).startsWith( "x-kbibtex-encoding=" ) )
+                        {
+                            m_encoding = commentText.mid( 19 );
+                            qDebug( "Switching encoding to <%s>", m_encoding.latin1() );
+                            parameterCommentsList.append( comment );
+                        }
+                        else
+                            remainingList.append( *it );
+                    }
                 }
             }
         }
 
+        int totalElements = ( int ) bibtexfile->count();
+        int currentPos = 0;
+
+        const char *encodingTo = m_encoding == "latex" ? "utf-8\0" : m_encoding.append( "\0" ).ascii();
+        m_iconvHandle = iconv_open( encodingTo, "utf-8" );
+
+        /** before anything else, write parameter comments */
+        for ( QValueList<Comment*>::iterator it = parameterCommentsList.begin(); it != parameterCommentsList.end() && result && !cancelFlag; it++ )
+        {
+            result &= writeComment( *iodevice, *it );
+            emit progress( ++currentPos, totalElements );
+        }
+
         /** first, write preambles and strings (macros) at the beginning */
         for ( QValueList<Preamble*>::iterator it = preambleList.begin(); it != preambleList.end() && result && !cancelFlag; it++ )
-            result &= writePreamble( stream, *it );
+        {
+            result &= writePreamble( *iodevice, *it );
+            emit progress( ++currentPos, totalElements );
+        }
 
         for ( QValueList<Macro*>::iterator it = macroList.begin(); it != macroList.end() && result && !cancelFlag; it++ )
-            result &= writeMacro( stream, *it );
+        {
+            result &= writeMacro( *iodevice, *it );
+            emit progress( ++currentPos, totalElements );
+        }
 
         /** second, write cross-referencing elements */
         for ( QValueList<Entry*>::iterator it = crossRefingEntryList.begin(); it != crossRefingEntryList.end() && result && !cancelFlag; it++ )
-            result &= writeEntry( stream, *it );
+        {
+            result &= writeEntry( *iodevice, *it );
+            emit progress( ++currentPos, totalElements );
+        }
 
         /** third, write remaining elements */
         for ( QValueList<Element*>::iterator it = remainingList.begin(); it != remainingList.end() && result && !cancelFlag; it++ )
         {
             Entry *entry = dynamic_cast<Entry*>( *it );
             if ( entry != NULL )
-                result &= writeEntry( stream, entry );
+                result &= writeEntry( *iodevice, entry );
             else
             {
                 Comment *comment = dynamic_cast<Comment*>( *it );
                 if ( comment != NULL )
-                    result &= writeComment( stream, comment );
+                    result &= writeComment( *iodevice, comment );
             }
+            emit progress( ++currentPos, totalElements );
         }
 
+        iconv_close( m_iconvHandle );
+        m_mutex.unlock();
         return result && !cancelFlag;
     }
 
     bool FileExporterBibTeX::save( QIODevice* iodevice, const Element* element, QStringList * /*errorLog*/ )
     {
+        m_mutex.lock();
         bool result = FALSE;
-        QTextStream stream( iodevice );
-        if ( m_encoding == File::encUTF8 )
-            stream.setEncoding( QTextStream::UnicodeUTF8 );
+
+        const char *encodingTo = m_encoding == "latex" ? "utf-8\0" : m_encoding.append( "\0" ).ascii();
+        m_iconvHandle = iconv_open( encodingTo, "utf-8" );
 
         const Entry *entry = dynamic_cast<const Entry*>( element );
         if ( entry != NULL )
-            result |= writeEntry( stream, entry );
+            result |= writeEntry( *iodevice, entry );
         else
         {
             const Macro * macro = dynamic_cast<const Macro*>( element );
             if ( macro != NULL )
-                result |= writeMacro( stream, macro );
+                result |= writeMacro( *iodevice, macro );
             else
             {
                 const Comment * comment = dynamic_cast<const Comment*>( element );
                 if ( comment != NULL )
-                    result |= writeComment( stream, comment );
+                    result |= writeComment( *iodevice, comment );
                 else
                 {
                     const Preamble * preamble = dynamic_cast<const Preamble*>( element );
                     if ( preamble != NULL )
-                        result |= writePreamble( stream, preamble );
+                        result |= writePreamble( *iodevice, preamble );
                 }
             }
         }
 
+        iconv_close( m_iconvHandle );
+        m_mutex.unlock();
         return result && !cancelFlag;
     }
 
@@ -146,9 +185,9 @@ namespace BibTeX
         cancelFlag = TRUE;
     }
 
-    bool FileExporterBibTeX::writeEntry( QTextStream &stream, const Entry* entry )
+    bool FileExporterBibTeX::writeEntry( QIODevice &device, const Entry* entry )
     {
-        stream << "@" << applyKeywordCasing( entry->entryTypeString() ) << "{ " << entry->id();
+        writeString( device, QString( "@%1{ %2" ).arg( applyKeywordCasing( entry->entryTypeString() ) ).arg( entry->id() ) );
 
         for ( Entry::EntryFields::ConstIterator it = entry->begin(); it != entry->end(); ++it )
         {
@@ -156,70 +195,87 @@ namespace BibTeX
             QString text = valueToString( field->value(), field->fieldType() );
             if ( m_protectCasing && dynamic_cast<BibTeX::PlainText*>( field->value()->items.first() ) != NULL && ( field->fieldType() == EntryField::ftTitle || field->fieldType() == EntryField::ftBookTitle || field->fieldType() == EntryField::ftSeries ) )
                 addProtectiveCasing( text );
-            stream << ',' << endl << '\t' << field->fieldTypeName() << " = " << text;
+            writeString( device, QString( ",\n\t%1 = %2" ).arg( field->fieldTypeName() ).arg( text ) );
         }
-        stream << endl << "}" << endl << endl;
+        writeString( device, "\n}\n\n" );
         return TRUE;
     }
 
-    bool FileExporterBibTeX::writeMacro( QTextStream &stream, const Macro *macro )
+    bool FileExporterBibTeX::writeMacro( QIODevice &device, const Macro *macro )
     {
         QString text = valueToString( macro->value() );
         if ( m_protectCasing )
             addProtectiveCasing( text );
 
-        stream << "@" << applyKeywordCasing( "String" ) << "{ " << macro->key() << " = " << text << " }" << endl << endl;
+        writeString( device, QString( "@%1{ %2 = %3 }\n\n" ).arg( applyKeywordCasing( "String" ) ).arg( macro->key() ).arg( text ) );
 
         return TRUE;
     }
 
-    bool FileExporterBibTeX::writeComment( QTextStream &stream, const Comment *comment )
+    bool FileExporterBibTeX::writeComment( QIODevice &device, const Comment *comment )
     {
         if ( !comment->useCommand() )
         {
             QString text = comment->text() ;
             escapeLaTeXChars( text );
 
-            switch ( m_encoding )
-            {
-            case File::encLaTeX:
+            if ( m_encoding == "latex" )
                 text = EncoderLaTeX::currentEncoderLaTeX() ->encode( text );
-                break;
-            default:
-                stream.setEncoding( QTextStream::UnicodeUTF8 );
-            }
+
             QStringList commentLines = QStringList::split( '\n', text );
             for ( QStringList::Iterator it = commentLines.begin(); it != commentLines.end(); it++ )
             {
-                stream << *it << endl;
+                writeString( device, ( *it ).append( "\n" ) );
             }
-            stream << endl;
+            writeString( device, "\n" );
         }
         else
         {
             QString text = comment->text() ;
             escapeLaTeXChars( text );
 
-            switch ( m_encoding )
-            {
-            case File::encLaTeX:
+            if ( m_encoding == "latex" )
                 text = EncoderLaTeX::currentEncoderLaTeX() ->encode( text );
-                break;
-            default:
-                stream.setEncoding( QTextStream::UnicodeUTF8 );
-            }
-            stream << "@" << applyKeywordCasing( "Comment" ) << "{" << text << "}" << endl << endl;
+            writeString( device, QString( "@%1{%2}\n\n" ).arg( applyKeywordCasing( "Comment" ) ).arg( text ) );
         }
         return TRUE;
     }
 
-    bool FileExporterBibTeX::writePreamble( QTextStream &stream, const Preamble* preamble )
+    bool FileExporterBibTeX::writePreamble( QIODevice &device, const Preamble* preamble )
     {
-        stream << "@" << applyKeywordCasing( "Preamble" ) << "{" << valueToString( preamble->value() )  << "}" << endl << endl;
+        writeString( device, QString( "@%1{%2}\n\n" ).arg( applyKeywordCasing( "Preamble" ) ).arg( valueToString( preamble->value() ) ) );
 
         return TRUE;
     }
 
+    bool FileExporterBibTeX::writeString( QIODevice &device, const QString& text )
+    {
+        const int encodeMaxTries = 1024;
+        QString workingText = text;
+        size_t utf8datasize = 1;
+        int i;
+        for ( i = 0; i < encodeMaxTries && utf8datasize != 0; ++i )
+        {
+            QCString utf8 = workingText.utf8();
+            char *utf8data = utf8.data();
+            utf8datasize = utf8.length();
+            char *outputdata = m_iconvBuffer;
+            size_t outputdatasize = m_iconvBufferSize;
+            size_t result = iconv( m_iconvHandle, &utf8data, &utf8datasize, &outputdata, &outputdatasize );
+
+            if ( device.writeBlock( m_iconvBuffer, m_iconvBufferSize - outputdatasize ) != ( int )( m_iconvBufferSize - outputdatasize ) )
+                return false;
+
+            if ( result != 0 )
+            {
+                workingText = QString::fromUtf8( utf8data, utf8datasize );
+                workingText = EncoderLaTeX::currentEncoderLaTeX()->encode( workingText, workingText[0] );
+            }
+        }
+
+        return i < encodeMaxTries;
+    }
+
     void FileExporterBibTeX::setStringDelimiter( const QChar& stringOpenDelimiter, const QChar& stringCloseDelimiter )
     {
         m_stringOpenDelimiter = stringOpenDelimiter;
@@ -231,7 +287,7 @@ namespace BibTeX
         m_keywordCasing = keywordCasing;
     }
 
-    void FileExporterBibTeX::setEncoding( File::Encoding encoding )
+    void FileExporterBibTeX::setEncoding( const QString& encoding )
     {
         m_encoding = encoding;
     }
@@ -314,16 +370,8 @@ namespace BibTeX
 
                 escapeLaTeXChars( text );
 
-                switch ( m_encoding )
-                {
-                case File::encLaTeX:
+                if ( m_encoding == "latex" )
                     text = encoder->encodeSpecialized( text, fieldType );
-                    break;
-                default:
-                    {
-                        // nothing
-                    }
-                }
 
                 /** if the text to save contains a quote char ("),
                   * force string delimiters to be curly brackets,
@@ -332,7 +380,7 @@ namespace BibTeX
                   */
                 QChar stringOpenDelimiter = m_stringOpenDelimiter;
                 QChar stringCloseDelimiter = m_stringCloseDelimiter;
-                if ( result.contains( '"' ) && ( m_stringOpenDelimiter == '"' || m_stringCloseDelimiter == '"' ) )
+                if ( text.contains( '"' ) && ( m_stringOpenDelimiter == '"' || m_stringCloseDelimiter == '"' ) )
                 {
                     stringOpenDelimiter = '{';
                     stringCloseDelimiter = '}';
