@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2008 by Thomas Fischer                             *
+*   Copyright (C) 2004-2009 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -19,7 +19,6 @@
 ***************************************************************************/
 #include <qiodevice.h>
 #include <qregexp.h>
-#include <qtextstream.h>
 #include <qapplication.h>
 
 #include <file.h>
@@ -33,32 +32,78 @@
 
 #include "fileimporterbibtex.h"
 
+#define max(a,b)  ((a)<(b)?(b):(a))
+
 namespace BibTeX
 {
     const QString extraAlphaNumChars = QString( "?'`-_:.+/$\\\"&" );
+    const QRegExp htmlRegExp = QRegExp( "</?(a|pre)[^>]*>", false );
 
-    FileImporterBibTeX::FileImporterBibTeX( bool personFirstNameFirst ) : FileImporter(), m_personFirstNameFirst( personFirstNameFirst ), m_currentChar( ' ' ), m_ignoreComments( FALSE )
+    FileImporterBibTeX::FileImporterBibTeX( bool personFirstNameFirst, QString encoding ) : FileImporter(), m_personFirstNameFirst( personFirstNameFirst ), m_currentChar( ' ' ), m_ignoreComments( FALSE ), m_lineBufferSize( 4096 ), m_encoding( encoding )
     {
         cancelFlag = FALSE;
+        m_lineBuffer = new char[m_lineBufferSize];
         m_textStream = NULL;
     }
 
 
     FileImporterBibTeX::~FileImporterBibTeX()
     {
-        // nothing
+        delete[] m_lineBuffer;
     }
 
     File* FileImporterBibTeX::load( QIODevice *iodevice )
     {
+        m_mutex.lock();
         cancelFlag = FALSE;
-        QTextStream rawTextStream( iodevice );
-        rawTextStream.setEncoding( QTextStream::UnicodeUTF8 );
-        QString rawText = rawTextStream.read();
+
+        QString rawText;
+        const char *encodingFrom = m_encoding == "latex" ? "utf-8\0" : m_encoding.append( "\0" ).ascii();
+        iconv_t iconvHandle = iconv_open( "utf-8", encodingFrom );
+        char *convertedLine = new char[m_lineBufferSize * 4];
+        int len;
+        bool encodingOk = true;
+        while ( encodingOk && iodevice->isReadable() && ( len = iodevice->readLine( m_lineBuffer, m_lineBufferSize ) ) > 0 )
+        {
+            evaluateParameterComments( iconvHandle, m_lineBuffer );
+
+            char *raw = m_lineBuffer;
+            char *enc = convertedLine;
+            size_t encLen = m_lineBufferSize, rawLen = ( size_t )len;
+            size_t result = iconv( iconvHandle, &raw, &rawLen, &enc, &encLen );
+
+            qApp->processEvents();
+
+            if ( result != 0 )
+            {
+                QString problematic = QString( m_lineBuffer ).mid( max( 0, m_lineBufferSize - encLen - 15 ), 30 );
+                if ( problematic.isNull() || problematic.isEmpty() ) problematic = QString( m_lineBuffer );
+                qDebug( "iconv resulted in error code %i for source encoding %s, maybe file is in different encoding? Problem is somewhere here: \"%s\"", result, encodingFrom, problematic.latin1() );
+                encodingOk = false;
+                break;
+            }
+            if ( rawLen > 0 )
+            {
+                qDebug( "iconv could not convert complete string, only %i out of %i chars", len - rawLen, len );
+                encodingOk = false;
+                break;
+            }
+            enc[0] = '\0';
+            QString line = QString::fromUtf8( convertedLine );
+            rawText.append( line );
+        }
+        iconv_close( iconvHandle );
+        delete[] convertedLine;
+
+        if ( !encodingOk )
+        {
+            qDebug( "Decoding failed, cannot load file. Please fix encoding manually." );
+            m_mutex.unlock();
+            return NULL;
+        }
 
         /** Cleaning up code comming from DBLP */
-        rawText = rawText.replace( "<pre>", "\n\n" ).replace( "</pre>", "\n\n" ).remove( "<a href=\"http://www.informatik.uni-trier.de/~ley/db/about/bibtex.html\">DBLP</a>:" );
-
+        rawText = rawText.replace( htmlRegExp, "" );
         rawText = EncoderLaTeX::currentEncoderLaTeX() ->decode( rawText );
         unescapeLaTeXChars( rawText );
         m_textStream = new QTextStream( rawText, IO_ReadOnly );
@@ -81,7 +126,7 @@ namespace BibTeX
             }
             qApp->processEvents();
         }
-        emit progress( 100, 100 );
+        emit progress( streamDevice->size(), streamDevice->size() );
 
         if ( cancelFlag )
         {
@@ -91,6 +136,8 @@ namespace BibTeX
         }
 
         delete m_textStream;
+
+        m_mutex.unlock();
         return result;
     }
 
@@ -142,7 +189,7 @@ namespace BibTeX
 
     Comment *FileImporterBibTeX::readCommentElement()
     {
-        while ( m_currentChar != '{' && m_currentChar != '(' )
+        while ( m_currentChar != '{' && m_currentChar != '(' && !m_textStream->atEnd() )
             *m_textStream >> m_currentChar;
 
         return new Comment( readBracketString( m_currentChar ), TRUE );
@@ -259,18 +306,31 @@ namespace BibTeX
 
             QString fieldTypeName = readSimpleString();
             token = nextToken();
-            if ( token == tBracketClose )
+            if ( fieldTypeName == QString::null || token == tBracketClose )
             {
                 // entry is buggy, but we still accept it
                 break;
             }
             else if ( token != tAssign )
             {
-                qDebug( "Error in parsing entry '%s': Assign symbol (=) expected", key.latin1() );
+                qDebug( "Error in parsing entry '%s': Assign symbol (=) expected after field name '%s'", key.latin1(), fieldTypeName.latin1() );
                 delete entry;
                 return NULL;
             }
 
+            /** check for duplicate fields */
+            if ( entry->getField( fieldTypeName ) != NULL )
+            {
+                int i = 1;
+                QString appendix = QString::number( i );
+                while ( entry->getField( fieldTypeName + appendix ) != NULL )
+                {
+                    ++i;
+                    appendix = QString::number( i );
+                }
+                fieldTypeName += appendix;
+            }
+
             EntryField *entryField = new EntryField( fieldTypeName );
 
             token = readValue( entryField->value(), entryField->fieldType() );
@@ -289,7 +349,7 @@ namespace BibTeX
 
         Token curToken = tUnknown;
 
-        while ( m_currentChar.isSpace() )
+        while (( m_currentChar.isSpace() || m_currentChar == '\t' ) && !m_textStream->atEnd() )
             *m_textStream >> m_currentChar;
 
         switch ( m_currentChar.latin1() )
@@ -454,38 +514,38 @@ namespace BibTeX
             switch ( fieldType )
             {
             case EntryField::ftKeywords:
-                {
-                    if ( isStringKey )
-                        qDebug( "WARNING: Cannot handle keywords that are macros" );
-                    else
-                        value->items.append( new KeywordContainer( text ) );
-                }
-                break;
+            {
+                if ( isStringKey )
+                    qDebug( "WARNING: Cannot handle keywords that are macros" );
+                else
+                    value->items.append( new KeywordContainer( text ) );
+            }
+            break;
             case EntryField::ftAuthor:
             case EntryField::ftEditor:
+            {
+                if ( isStringKey )
+                    qDebug( "WARNING: Cannot handle authors/editors that are macros" );
+                else
                 {
-                    if ( isStringKey )
-                        qDebug( "WARNING: Cannot handle authors/editors that are macros" );
-                    else
-                    {
-                        QStringList persons;
-                        splitPersons( text, persons );
-                        PersonContainer *container = new PersonContainer( m_personFirstNameFirst );
-                        for ( QStringList::ConstIterator pit = persons.constBegin(); pit != persons.constEnd(); ++pit )
-                            container->persons.append( new Person( *pit, m_personFirstNameFirst ) );
-                        value->items.append( container );
-                    }
+                    QStringList persons;
+                    splitPersons( text, persons );
+                    PersonContainer *container = new PersonContainer( m_personFirstNameFirst );
+                    for ( QStringList::ConstIterator pit = persons.constBegin(); pit != persons.constEnd(); ++pit )
+                        container->persons.append( new Person( *pit, m_personFirstNameFirst ) );
+                    value->items.append( container );
                 }
-                break;
+            }
+            break;
             case EntryField::ftPages:
                 text.replace( QRegExp( "\\s*--?\\s*" ), QChar( 0x2013 ) );
             default:
-                {
-                    if ( isStringKey )
-                        value->items.append( new MacroKey( text ) );
-                    else
-                        value->items.append( new BibTeX::PlainText( text ) );
-                }
+            {
+                if ( isStringKey )
+                    value->items.append( new MacroKey( text ) );
+                else
+                    value->items.append( new BibTeX::PlainText( text ) );
+            }
             }
 
             token = nextToken();
@@ -532,4 +592,23 @@ namespace BibTeX
         wordList.append( word );
         persons.append( wordList.join( " " ) );
     }
+
+    void FileImporterBibTeX::evaluateParameterComments( iconv_t &iconvHandle, const char *cline )
+    {
+        /** simple preliminary checks before expensive conversion to QString */
+        if ( cline[0] == '@' && cline[1] == 'c' )
+        {
+            QString line = QString( cline ).lower();
+            /** check if this file requests a special encoding */
+            if ( line.startsWith( "@comment{x-kbibtex-encoding=" ) && line.endsWith( "}\n" ) )
+            {
+                QString newEncoding = line.mid( 28, line.length() - 30 );
+                qDebug( "x-kbibtex-encoding=<%s>", newEncoding.latin1() );
+                if ( newEncoding == "latex" ) newEncoding = "utf-8";
+                iconv_close( iconvHandle );
+                iconvHandle = iconv_open( "utf-8", newEncoding.append( '\0' ).ascii() );
+            }
+        }
+    }
+
 }
