@@ -20,6 +20,8 @@
 #ifndef BIBTEXFILEIMPORTERBIBTEX_H
 #define BIBTEXFILEIMPORTERBIBTEX_H
 
+#include <iconv.h>
+
 #include <fileimporter.h>
 #include <entryfield.h>
 
@@ -39,7 +41,7 @@ namespace BibTeX
     class FileImporterBibTeX : public FileImporter
     {
     public:
-        FileImporterBibTeX( bool personFirstNameFirst = FALSE );
+        FileImporterBibTeX( bool personFirstNameFirst = FALSE, QString encoding = "latex" );
         ~FileImporterBibTeX();
 
         File* load( QIODevice *iodevice );
@@ -61,6 +63,9 @@ namespace BibTeX
         bool m_personFirstNameFirst;
         QChar m_currentChar;
         bool m_ignoreComments;
+        char *m_lineBuffer;
+        const int m_lineBufferSize;
+        QString m_encoding;
 
         Comment *readCommentElement();
         Comment *readPlainCommentElement();
@@ -78,6 +83,7 @@ namespace BibTeX
 
         void unescapeLaTeXChars( QString &text );
         void splitPersons( const QString& test, QStringList &persons );
+        void evaluateParameterComments( iconv_t &iconvHandle, const char *line );
     };
 
 }
