@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2009 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -39,14 +39,14 @@
 static const char description[] =
     I18N_NOOP( "A BibTeX editor for KDE" );
 
-static const char version[] = "0.2";
+static const char version[] = "0.2.2";
 
 static KCmdLineOptions options[] =
-    {
-        { "+[URL]", I18N_NOOP( "Document to open." ), 0
-        },
-        KCmdLineLastOption
-    };
+{
+    { "+[URL]", I18N_NOOP( "Document to open." ), 0
+    },
+    KCmdLineLastOption
+};
 
 #ifdef UNIQUEAPP
 class KBibTeXApplication: public KUniqueApplication
@@ -92,7 +92,7 @@ public:
                     else
                     {
                         kdDebug() << "Couldn't open file " << args->url( i ).path() << endl;
-                        KMessageBox::error( NULL, i18n( "Could not open file %1." ).arg( args->url( i ).path() ) );
+                        KMessageBox::error( NULL, i18n( "Could not open file '%1'." ).arg( args->url( i ).path() ) );
                     }
                 }
             }
@@ -109,8 +109,9 @@ public:
 int main( int argc, char **argv )
 {
     KAboutData about( "kbibtex", I18N_NOOP( "KBibTeX" ), version, description,
-                      KAboutData::License_GPL, "(C) 2004-2007 Thomas Fischer", 0, "http://www.unix-ag.uni-kl.de/~fischer/kbibtex/", "fischer@unix-ag.uni-kl.de" );
+                      KAboutData::License_GPL, "(C) 2004-2009 Thomas Fischer", 0, "http://www.unix-ag.uni-kl.de/~fischer/kbibtex/", "fischer@unix-ag.uni-kl.de" );
     about.addAuthor( "Thomas Fischer", 0, "fischer@unix-ag.uni-kl.de" );
+    about.setTranslator( I18N_NOOP( "NAME OF TRANSLATORS" ), I18N_NOOP( "EMAIL OF TRANSLATORS" ) );
     KCmdLineArgs::init( argc, argv, &about );
     KCmdLineArgs::addCmdLineOptions( options );
 
