@@ -24,6 +24,7 @@
 
 #include <kxmlguifactory.h>
 
+#include <documentlistview.h>
 #include <entryfield.h>
 
 class QTimer;
@@ -34,7 +35,6 @@ class KComboBox;
 
 namespace KBibTeX
 {
-
     /**
     @author Thomas Fischer
     */
@@ -46,14 +46,14 @@ namespace KBibTeX
         ~SearchBar();
 
         void setFactory( KXMLGUIFactory *factory, KXMLGUIClient *client );
-        void setOnlineDatabaseSearchMenu( KPopupMenu* popupMenu );
         void restoreState();
 
     signals:
-        void doSearch( const QString&text, BibTeX::EntryField::FieldType fieldType );
+        void doSearch( const QString &text, BibTeX::Element::FilterType, BibTeX::EntryField::FieldType fieldType );
+        void onlineSearch();
 
     public slots:
-        void setSearch( const QString&text, BibTeX::EntryField::FieldType fieldType );
+        void setSearch( const QString&text, BibTeX::Element::FilterType, BibTeX::EntryField::FieldType fieldType );
 
     private:
         KPushButton *m_pushButtonClearSearchText;
@@ -61,6 +61,7 @@ namespace KBibTeX
         QTimer *m_timerInput;
         KPushButton *m_pushButtonAddElement;
         KPushButton *m_pushButtonSearchOnlineDatabases;
+        KComboBox *m_comboboxFilterType;
         KComboBox *m_comboboxRestrictTo;
 
         void setupGUI();
@@ -69,6 +70,7 @@ namespace KBibTeX
         void slotClear();
         void slotTimeout();
         void slotAnnounceDoSearch();
+        void slotKeyPressed();
     };
 
 }
