@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2008 by Thomas Fischer                             *
+*   Copyright (C) 2004-2009 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -42,9 +42,11 @@
 
 namespace KBibTeX
 {
-    Settings* Settings::staticSettings = new Settings();;
+    Settings* Settings::staticSettings = new Settings();
     QStringList Settings::m_lyxRcFileNames = QStringList::split( '|', QDir::home().canonicalPath() + "/.lyx/lyxrc" + '|' + QDir::home().canonicalPath() + "/.lyx/preferences" );
 
+    const QRegExp Settings::noIdChars( "[^-.:/+_a-zA-Z0-9]" );
+
     Settings::Settings()
     {
         checkExternalToolsAvailable();
@@ -56,7 +58,7 @@ namespace KBibTeX
 
         int numCompletions = BibTeX::EntryField::ftYear - BibTeX::EntryField::ftAbstract + 4;
         m_completion = new KCompletion * [ numCompletions ];
-        for ( int i = 0; i < numCompletions;i++ )
+        for ( int i = 0; i < numCompletions; ++i )
         {
             m_completion[ i ] = new KCompletion();
             m_completion[ i ] ->setIgnoreCase( TRUE );
@@ -70,7 +72,7 @@ namespace KBibTeX
     Settings::~Settings()
     {
         int numCompletions = BibTeX::EntryField::ftYear - BibTeX::EntryField::ftAbstract + 4;
-        for ( int i = 0; i < numCompletions;i++ )
+        for ( int i = 0; i < numCompletions; ++i )
             delete m_completion[ i ];
         delete[] m_completion;
         delete completionMacro;
@@ -89,15 +91,19 @@ namespace KBibTeX
     void Settings::load( KConfig * config )
     {
         config->setGroup( "FileIO" );
-        fileIO_Encoding = ( BibTeX::File::Encoding )( config->readNumEntry( "Encoding", ( int )( BibTeX::File::encLaTeX - 1 ) ) + 1 );
+        fileIO_Encoding = config->readEntry( "Encoding", "latex" );
+        if ( fileIO_Encoding.length() == 1 ) fileIO_Encoding = "latex"; // due to old scheme where digits were used
         fileIO_ExportLanguage = config->readEntry( "ExportLanguage", "english" );
         fileIO_ExportBibliographyStyle = config->readEntry( "ExportBibliographyStyle", "plain" );
         fileIO_ExporterHTML = ( BibTeX::FileExporterExternal::Exporter ) config->readNumEntry( "ExporterHTML", ( int ) BibTeX::FileExporterExternal::exporterNone );
         fileIO_BibtexStringOpenDelimiter = config->readEntry( "BibtexStringOpenDelimiter", "\"" ).at( 0 );
         fileIO_BibtexStringCloseDelimiter = config->readEntry( "BibtexStringCloseDelimiter", "\"" ).at( 0 );
-        fileIO_KeywordCasing = ( BibTeX::FileExporterBibTeX::KeywordCasing )config->readNumEntry( "KeywordCasing", ( int )BibTeX::FileExporterBibTeX::kcCamelCase );
+        fileIO_KeywordCasing = ( BibTeX::FileExporterBibTeX::KeywordCasing ) config->readNumEntry( "KeywordCasing", ( int ) BibTeX::FileExporterBibTeX::kcCamelCase );
         fileIO_EmbedFiles = config->readBoolEntry( "EmbedFiles", FALSE );
         fileIO_EnclosingCurlyBrackets = config->readBoolEntry( "EnclosingCurlyBrackets", FALSE );
+        fileIO_useBibUtils = config->readBoolEntry( "UseBibUtils", TRUE );
+        updateBib2Db5ClassPath( config->readEntry( "Bib2Db5BasePath", QString::null ) );
+        fileIO_NumberOfBackups = config->readNumEntry( "NumberOfBackups", 0 );
 
         config->setGroup( "Editing" );
         editing_SearchBarClearField = config->readBoolEntry( "SearchBarClearField", false );
@@ -116,8 +122,8 @@ namespace KBibTeX
         editing_SpecialFont = config->readFontEntry( "SpecialFont", &defaultFont );
         editing_UseSpecialFont = config->readBoolEntry( "UseSpecialFont", FALSE );
         editing_FirstNameFirst = config->readBoolEntry( "FirstNameFirst", FALSE );
-        editing_DocumentSearchPath = config->readEntry( "DocumentSearchPath", "" );
-        editing_DragAction = (DragAction)config->readNumEntry( "DragAction", ( int )COPYREFERENCE );
+        editing_DocumentSearchPaths = config->readListEntry( "DocumentSearchPath" );
+        editing_DragAction = ( DragAction ) config->readNumEntry( "DragAction", ( int ) COPYREFERENCE );
 
         if ( editing_MainListColumnsWidth.isEmpty() || editing_MainListColumnsIndex.isEmpty() )
         {
@@ -141,6 +147,7 @@ namespace KBibTeX
                 editing_MainListColumnsIndex.append( column++ );
             }
         }
+        editing_findDuplicatesSensitivity = config->readNumEntry( "FindDuplicatesSensitivity", 12 );
 
         config->setGroup( "SearchURLs" );
         searchURLs.clear();
@@ -202,20 +209,32 @@ namespace KBibTeX
             udif->inputType = ( inputtypes[i] == "single" ? FieldLineEdit::itSingleLine : FieldLineEdit::itMultiLine );
             userDefinedInputFields << udif;
         }
+
+        // web query history/default values
+        m_webQueryDefaults = config->entryMap( "WebQuery" );
+
+#ifdef HAVE_YAZ
+        z3950clearAll();
+        z3950loadUser( config );
+        z3950loadDefault();
+#endif // HAVE_YAZ
     }
 
     void Settings::save( KConfig * config )
     {
         config->setGroup( "FileIO" );
-        config->writeEntry( "Encoding", ( int ) fileIO_Encoding - 1 );
+        config->writeEntry( "Encoding", fileIO_Encoding );
         config->writeEntry( "ExportLanguage", fileIO_ExportLanguage );
         config->writeEntry( "ExportBibliographyStyle", fileIO_ExportBibliographyStyle );
         config->writeEntry( "ExporterHTML", ( int ) fileIO_ExporterHTML );
         config->writeEntry( "BibtexStringOpenDelimiter", QString( fileIO_BibtexStringOpenDelimiter ) );
         config->writeEntry( "BibtexStringCloseDelimiter", QString( fileIO_BibtexStringCloseDelimiter ) );
-        config->writeEntry( "KeywordCasing", ( int )fileIO_KeywordCasing );
+        config->writeEntry( "KeywordCasing", ( int ) fileIO_KeywordCasing );
         config->writeEntry( "EmbedFiles", fileIO_EmbedFiles );
         config->writeEntry( "EnclosingCurlyBrackets", fileIO_EnclosingCurlyBrackets );
+        config->writeEntry( "UseBibUtils", fileIO_useBibUtils );
+        config->writeEntry( "Bib2Db5BasePath", fileIO_bib2db5BasePath );
+        config->writeEntry( "NumberOfBackups", fileIO_NumberOfBackups );
 
         config->setGroup( "Editing" );
         config->writeEntry( "SearchBarClearField", editing_SearchBarClearField );
@@ -233,8 +252,9 @@ namespace KBibTeX
         config->writeEntry( "SpecialFont", editing_SpecialFont );
         config->writeEntry( "UseSpecialFont", editing_UseSpecialFont );
         config->writeEntry( "FirstNameFirst", editing_FirstNameFirst );
-        config->writeEntry( "DocumentSearchPath", editing_DocumentSearchPath );
+        config->writeEntry( "DocumentSearchPath", editing_DocumentSearchPaths );
         config->writeEntry( "DragAction", editing_DragAction );
+        config->writeEntry( "FindDuplicatesSensitivity", editing_findDuplicatesSensitivity );
 
         config->setGroup( "SearchURLs" );
         int i = 1;
@@ -270,6 +290,14 @@ namespace KBibTeX
         config->writeEntry( "Names", names );
         config->writeEntry( "Labels", labels );
         config->writeEntry( "InputTypes", inputtype );
+
+        config->setGroup( "WebQuery" );
+        for ( QMap<QString, QString>::Iterator it = m_webQueryDefaults.begin(); it != m_webQueryDefaults.end(); ++it )
+            config->writeEntry( it.key(), it.data() );
+
+#ifdef HAVE_YAZ
+        z3950saveUser( config );
+#endif // HAVE_YAZ
     }
 
     void Settings::checkExternalToolsAvailable()
@@ -278,30 +306,57 @@ namespace KBibTeX
         external_bibtex2htmlAvailable = checkExternalToolAvailable( "bibtex2html" );
         external_bib2xhtmlAvailable = checkExternalToolAvailable( "bib2xhtml" );
         external_latex2rtfAvailable = checkExternalToolAvailable( "latex2rtf" );
+        external_xml2bibAvailable = checkExternalToolAvailable( "xml2bib" );
+        external_end2xmlAvailable = checkExternalToolAvailable( "end2xml" );
+        external_ris2xmlAvailable = checkExternalToolAvailable( "ris2xml" );
+    }
+
+    bool Settings::updateBib2Db5ClassPath( const QString& newBasePath, bool testOnly )
+    {
+        QString bib2db5ClassPath = QString::null;
+        QDir bib2db5BaseDir( newBasePath );
+        QStringList list = bib2db5BaseDir.entryList( "antlr-runtime*.jar" );
+        if ( !list.isEmpty() )
+        {
+            bib2db5ClassPath = bib2db5BaseDir.absPath() + "/" + list[0];
+            list = bib2db5BaseDir.entryList( "bib2db5*.jar" );
+            if ( !list.isEmpty() )
+                bib2db5ClassPath += ":" + bib2db5BaseDir.absPath() + "/" + list[0];
+            else
+                bib2db5ClassPath = QString::null;
+        }
+        else
+            bib2db5ClassPath = QString::null;
+        if ( bib2db5ClassPath == QString::null )
+            kdDebug() << "Base path for bib2db5 is invalid (\"" << newBasePath << "\")" << endl;
+
+        if ( !testOnly )
+        {
+            fileIO_bib2db5BasePath = newBasePath;
+            fileIO_bib2db5ClassPath = bib2db5ClassPath;
+        }
+
+        return bib2db5ClassPath != QString::null;
     }
 
     bool Settings::checkExternalToolAvailable( const QString &binary )
     {
-        QProcess process( binary );
-        if ( !process.start() )
-            return FALSE;
-
-        if ( process.normalExit() )
-            return TRUE;
-
-        if ( process.isRunning() )
+        QProcess *process = new QProcess( binary );
+        bool ok = process->start();
+        ok &= process->normalExit();
+        if ( process->isRunning() )
         {
-            process.kill();
-            return TRUE;
+            process->kill();
+            ok = true;
         }
-
-        return FALSE;
+        delete process;
+        return ok;
     }
 
     QString Settings::detectLyXInPipe()
     {
-        QString result = "";
-        for ( QStringList::Iterator it = m_lyxRcFileNames.begin(); result.isEmpty() && it != m_lyxRcFileNames.end(); ++it )
+        QString result = QString::null;
+        for ( QStringList::Iterator it = m_lyxRcFileNames.begin(); result.isNull() && it != m_lyxRcFileNames.end(); ++it )
         {
             QString lyxRcFileName = *it;
             QFile lyxRcFile( lyxRcFileName );
@@ -310,7 +365,7 @@ namespace KBibTeX
             if ( lyxRcFile.exists() && lyxRcFile.open( IO_ReadOnly ) )
             {
                 QTextStream input( &lyxRcFile );
-                while ( result.isEmpty() && !input.atEnd() )
+                while ( result.isNull() && !input.atEnd() )
                 {
                     QString line = input.readLine();
                     if ( line.startsWith( "\\serverpipe ", FALSE ) )
@@ -329,34 +384,44 @@ namespace KBibTeX
             }
         }
 
-        if ( result.isEmpty() )
+        if ( result.isNull() )
         {
             result = QDir::home().canonicalPath() + "/.lyx/lyxpipe.in";
             kdDebug() << "detectLyXInPipe:  testing " << result << endl;
             if ( !QFile::exists( result ) )
-                result = "";
+                result = QString::null;
         }
 
-        if ( result.isEmpty() )
+        if ( result.isNull() )
         {
             result = QDir::home().canonicalPath() + "/.lyx/.lyxpipe.in";
             kdDebug() << "detectLyXInPipe:  testing " << result << endl;
             if ( !QFile::exists( result ) )
-                result = "";
+                result = QString::null;
         }
 
-        if ( result.isEmpty() )
+        if ( result.isNull() )
         {
             result = QDir::home().canonicalPath() + "/.lyxpipe.in";
             kdDebug() << "detectLyXInPipe:  testing " << result << endl;
             if ( !QFile::exists( result ) )
-                result = "";
+                result = QString::null;
         }
 
-        kdDebug() << "detectLyXInPipe:  using " << ( result.isEmpty() ? "<EMPTY>" : result ) << endl;
+        kdDebug() << "detectLyXInPipe:  using " << ( result.isNull() ? "<INVALID>" : result ) << endl;
         return result;
     }
 
+    QString Settings::getWebQueryDefault( const QString &key )
+    {
+        return m_webQueryDefaults[key];
+    }
+
+    void Settings::setWebQueryDefault( const QString &key, const QString& value )
+    {
+        m_webQueryDefaults[key] = value;
+    }
+
     void Settings::addToCompletion( BibTeX::File *file )
     {
         for ( BibTeX::File::ElementList::iterator it = file->begin(); it != file->end(); it++ )
@@ -622,6 +687,45 @@ namespace KBibTeX
         }
     }
 
+    QString Settings::entryTypeToI18NString( const BibTeX::Entry::EntryType entryType )
+    {
+        switch ( entryType )
+        {
+        case BibTeX::Entry::etArticle:
+            return QString( i18n( "Article" ) );
+        case BibTeX::Entry::etBook:
+            return QString( i18n( "Book" ) );
+        case BibTeX::Entry::etBooklet:
+            return QString( i18n( "Booklet" ) );
+        case BibTeX::Entry::etCollection:
+            return QString( i18n( "Collection" ) );
+        case BibTeX::Entry::etElectronic:
+            return QString( i18n( "Electronic" ) );
+        case BibTeX::Entry::etInBook:
+            return QString( i18n( "InBook" ) );
+        case BibTeX::Entry::etInCollection:
+            return QString( i18n( "InCollection" ) );
+        case BibTeX::Entry::etInProceedings:
+            return QString( i18n( "InProceedings" ) );
+        case BibTeX::Entry::etManual:
+            return QString( i18n( "Manual" ) );
+        case BibTeX::Entry::etMastersThesis:
+            return QString( i18n( "MastersThesis" ) );
+        case BibTeX::Entry::etMisc:
+            return QString( i18n( "Misc" ) );
+        case BibTeX::Entry::etPhDThesis:
+            return QString( i18n( "PhDThesis" ) );
+        case BibTeX::Entry::etProceedings:
+            return QString( i18n( "Proceedings" ) );
+        case BibTeX::Entry::etTechReport:
+            return QString( i18n( "TechReport" ) );
+        case BibTeX::Entry::etUnpublished:
+            return QString( i18n( "Unpublished" ) );
+        default:
+            return QString( i18n( "Unknown" ) );
+        }
+    }
+
     KURL Settings::doiURL( const QString& doiText )
     {
         KURL result( doiText );
@@ -665,22 +769,29 @@ namespace KBibTeX
 
     KURL Settings::locateFile( const QString& filename, const QString& bibTeXFileName, QWidget *window )
     {
-        KURL url( filename );
+        QString _filename = filename;
+        QString userHome = QString( getenv( "HOME" ) );
+        if ( _filename.contains( "~/" ) && !userHome.isEmpty() ) _filename = _filename.replace( "~/", userHome.append( "/" ) );
+
+        KURL url( _filename );
         if ( url.isValid() && ( !url.isLocalFile() || KIO::NetAccess::exists( url, TRUE, window ) ) )
             return url;
 
         if ( bibTeXFileName != QString::null )
         {
             QString path = KURL( bibTeXFileName ).directory( FALSE, FALSE );
-            url = KURL( path + "/" + filename );
+            url = KURL( path + "/" + _filename );
             if ( url.isValid() && KIO::NetAccess::exists( url, TRUE, window ) )
                 return url;
         }
 
         Settings* settings = self( NULL );
-        url = KURL( settings->editing_DocumentSearchPath + "/" + filename );
-        if ( url.isValid() && KIO::NetAccess::exists( url, TRUE, window ) )
-            return url;
+        for ( QStringList::Iterator it = settings->editing_DocumentSearchPaths.begin(); it != settings->editing_DocumentSearchPaths.end(); ++it )
+        {
+            url = KURL( *it + "/" + _filename );
+            if ( url.isValid() && KIO::NetAccess::exists( url, TRUE, window ) )
+                return url;
+        }
 
         return KURL();
     }
@@ -690,4 +801,143 @@ namespace KBibTeX
         return locateFile( filename, currentBibTeXFile == NULL ? QString::null : currentBibTeXFile->fileName, window );
     }
 
+#ifdef HAVE_YAZ
+    void Settings::z3950clearAll()
+    {
+        z3950_ServerList.clear();
+    }
+
+    void Settings::z3950loadDefault()
+    {
+        QString z3950serverConfigFile = locate( "appdata", "z3950-servers.cfg" );
+        if ( !z3950serverConfigFile.isEmpty() )
+        {
+            KConfig z3950serverConfig( z3950serverConfigFile, true, false );
+            QStringList serverList = z3950serverConfig.groupList();
+            for ( QStringList::ConstIterator it = serverList.begin(); it != serverList.end(); ++it )
+                if ( !z3950_ServerList.contains( *it ) )
+                {
+                    z3950serverConfig.setGroup( *it );
+
+                    Z3950Server server;
+                    server.name = z3950serverConfig.readEntry( "Name" );
+                    server.host = z3950serverConfig.readEntry( "Host" );
+                    server.port = z3950serverConfig.readNumEntry( "Port", 2100 );
+                    server.database = z3950serverConfig.readEntry( "Database" );
+                    server.charset = z3950serverConfig.readEntry( "Charset" );
+                    server.syntax = z3950serverConfig.readEntry( "Syntax" );
+                    server.user = z3950serverConfig.readEntry( "User" );
+                    server.password = z3950serverConfig.readEntry( "Password" );
+                    server.locale = z3950serverConfig.readEntry( "Locale" );
+
+                    z3950_ServerList[*it] = server;
+                }
+        }
+    }
+
+    void Settings::z3950loadUser( KConfig * config )
+    {
+        config->setGroup( "Z3950Server" );
+        int i = 0;
+        QString key = config->readEntry( QString( "Key%1" ).arg( i ) );
+        while ( key != QString::null )
+        {
+            Z3950Server server;
+            server.name = config->readEntry( QString( "Name%1" ).arg( i ) );
+            server.host = config->readEntry( QString( "Host%1" ).arg( i ) );
+            server.port = config->readNumEntry( QString( "Port%1" ).arg( i ), 2100 );
+            server.database = config->readEntry( QString( "Database%1" ).arg( i ) );
+            server.charset = config->readEntry( QString( "Charset%1" ).arg( i ) );
+            server.syntax = config->readEntry( QString( "Syntax%1" ).arg( i ) );
+            server.user = config->readEntry( QString( "User%1" ).arg( i ) );
+            server.password = config->readEntry( QString( "Password%1" ).arg( i ) );
+            server.locale = config->readEntry( QString( "Locale%1" ).arg( i ) );
+            z3950_ServerList[key] = server;
+
+            ++i;
+            key = config->readEntry( QString( "Key%1" ).arg( i ) );
+        }
+    }
+
+    void Settings::z3950saveUser( KConfig *config )
+    {
+        config->deleteGroup( "Z3950Server" );
+        config->setGroup( "Z3950Server" );
+        int i = 0;
+
+        QString z3950serverConfigFile = locate( "appdata", "z3950-servers.cfg" );
+        if ( !z3950serverConfigFile.isEmpty() )
+        {
+            KConfig z3950serverConfig( z3950serverConfigFile, true, false );
+            for ( QMap<QString, Z3950Server>::Iterator it = z3950_ServerList.begin(); it != z3950_ServerList.end(); ++it )
+            {
+                bool notInDefault = !z3950serverConfig.hasGroup( it.key() );
+                if ( !notInDefault )
+                {
+                    z3950serverConfig.setGroup( it.key() );
+                    notInDefault = z3950serverConfig.readEntry( "Name", it.data().name ) != it.data().name  || z3950serverConfig.readEntry( "Host", it.data().host ) != it.data().host || z3950serverConfig.readNumEntry( "Port", it.data().port ) != it.data().port || z3950serverConfig.readEntry( "Database", it.data().database ) != it.data().database || z3950serverConfig.readEntry( "Charset", it.data().charset ) != it.data().charset || z3950serverConfig.readEntry( "Syntax", it.data().syntax ) != it.data().syntax || z3950serverConfig.readEntry( "User", it.data().user ) != it.data().user || z3950serverConfig.readEntry( "Password", it.data().password ) != it.data().password || z3950serverConfig.readEntry( "Locale", it.data().locale ) != it.data().locale || z3950serverConfig.readEntry( "User", it.data().user ) != it.data().user;
+                }
+
+                if ( notInDefault )
+                {
+                    config->writeEntry( QString( "Key%1" ).arg( i ),  it.key() );
+                    config->writeEntry( QString( "Name%1" ).arg( i ),  it.data().name );
+                    config->writeEntry( QString( "Host%1" ).arg( i ),  it.data().host );
+                    config->writeEntry( QString( "Port%1" ).arg( i ),  it.data().port );
+                    config->writeEntry( QString( "Database%1" ).arg( i ),  it.data().database );
+                    config->writeEntry( QString( "Charset%1" ).arg( i ),  it.data().charset );
+                    config->writeEntry( QString( "Syntax%1" ).arg( i ),  it.data().syntax );
+                    config->writeEntry( QString( "User%1" ).arg( i ),  it.data().user );
+                    config->writeEntry( QString( "Password%1" ).arg( i ),  it.data().password );
+                    config->writeEntry( QString( "Locale%1" ).arg( i ),  it.data().locale );
+                    ++i;
+                }
+            }
+        }
+    }
+
+#endif // HAVE_YAZ
+
+    /** Create backup of file as specified in fileIO_NumberOfBackups */
+    bool Settings::createBackup( const KURL &url, QWidget *window )
+    {
+        QString basename = url.prettyURL();
+
+        qDebug( "Making %i backups of URL %s", fileIO_NumberOfBackups, basename.latin1() );
+
+        for ( int i = fileIO_NumberOfBackups; i > 2; --i )
+        {
+            KURL backupN( basename + "~" + QString::number( i ) );
+            KURL backupNmm( basename + "~" + QString::number( i - 1 ) );
+            if ( KIO::NetAccess::exists( backupNmm, true, window ) )
+            {
+                if ( !KIO::NetAccess::file_copy( backupNmm, backupN, -1, true, false, false ) )
+                    return false;
+            }
+        }
+
+        if ( fileIO_NumberOfBackups >= 2 )
+        {
+            KURL backupN( basename + "~2" );
+            KURL backupNmm( basename + "~" );
+            if ( KIO::NetAccess::exists( backupNmm, true, window ) )
+            {
+                if ( !KIO::NetAccess::file_copy( backupNmm, backupN, -1, true, false, false ) )
+                    return false;
+            }
+        }
+        if ( fileIO_NumberOfBackups >= 1 )
+        {
+            KURL backupN( basename + "~" );
+            KURL backupNmm( basename );
+            if ( KIO::NetAccess::exists( backupNmm, true, window ) )
+            {
+                if ( !KIO::NetAccess::file_copy( backupNmm, backupN, -1, true, false, false ) )
+                    return false;
+            }
+        }
+        return true;
+    }
+
+
 }
