@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2005 by Thomas Fischer                             *
+ *   Copyright (C) 2004-2009 by Thomas Fischer                             *
  *   fischer@unix-ag.uni-kl.de                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -216,7 +216,8 @@ namespace KBibTeX
                 item->setPixmap( 0, SmallIcon( "filter" ) );
                 m_defaultSuggestionItem = itemAbove;
             }
-            m_defaultSuggestionItem->setPixmap( 0, SmallIcon( "favorites" ) );
+            if ( m_defaultSuggestionItem != NULL )
+                m_defaultSuggestionItem->setPixmap( 0, SmallIcon( "favorites" ) );
         }
     }
 
@@ -242,7 +243,8 @@ namespace KBibTeX
                 item->setPixmap( 0, SmallIcon( "filter" ) );
                 m_defaultSuggestionItem = itemBelow;
             }
-            m_defaultSuggestionItem->setPixmap( 0, SmallIcon( "favorites" ) );
+            if ( m_defaultSuggestionItem != NULL )
+                m_defaultSuggestionItem->setPixmap( 0, SmallIcon( "favorites" ) );
         }
     }
 
@@ -283,6 +285,7 @@ namespace KBibTeX
         m_listIdSuggestions->setSorting( -1, FALSE );
         m_listIdSuggestions->addColumn( i18n( "Id Suggestions" ) );
         m_listIdSuggestions->header()->setClickEnabled( FALSE );
+        m_listIdSuggestions->setFullWidth( true );
         gridLayout->addMultiCellWidget( m_listIdSuggestions, 0, 6, 0, 0 );
         connect( m_listIdSuggestions, SIGNAL( selectionChanged() ), this, SLOT( updateGUI() ) );
         connect( m_listIdSuggestions, SIGNAL( currentChanged( QListViewItem * ) ), this, SLOT( updateGUI() ) );
