@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2005 by Thomas Fischer                             *
+ *   Copyright (C) 2004-2009 by Thomas Fischer                             *
  *   fischer@unix-ag.uni-kl.de                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,10 +18,11 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <qfile.h>
-#include <qregexp.h>
 #include <qbuffer.h>
+#include <qspinbox.h>
 
 #include <klocale.h>
+#include <klineedit.h>
 #include <kmessagebox.h>
 #include <kurl.h>
 
@@ -32,15 +33,35 @@
 
 namespace KBibTeX
 {
+    WebQueryArXivWidget::WebQueryArXivWidget( QWidget *parent, const char *name )
+            : WebQueryWidget( parent, name )
+    {
+        init();
 
-    WebQueryArXiv::WebQueryArXiv( QWidget* parent ): WebQuery( parent ), m_arXivServer( "www.arxiv.org" )
+        Settings *settings = Settings::self();
+        QString value = settings->getWebQueryDefault( "ArXiv" );
+        value = value == QString::null ? "" : value;
+        lineEditQuery->setText( value );
+        slotTextChanged( value, true );
+    }
+
+    WebQueryArXiv::WebQueryArXiv( QWidget* parent )
+            : WebQuery( parent ), m_arXivServer( "www.arxiv.org" ),
+            m_jourRef1( "^([a-zA-Z. ]+[a-zA-Z.])\\s*(\\d+)\\s+\\((\\d{4})\\)\\s+([0-9A-Z]+)(-([0-9A-Z]+))?$" ),
+            m_jourRef2( "^([a-zA-Z. ]+[a-zA-Z.]),\\s+Vol\\.?\\s+(\\d+)[,]?\\s+No\\.?\\s+(\\d+)\\s+\\((\\d{4})\\)[,]?\\s+(pp\\.\\s+)?(\\d+)(-(\\d+))?$" ),
+            m_jourRef3( "^([a-zA-Z. ]+),\\s+volume\\s+(\\d+),\\s+number\\s+(\\d+),\\s+pp\\.\\s+(\\d+)(-(\\d+))?,\\s+(\\d{4})$" ),
+            m_jourRef4( "^([a-zA-Z. ()]+)[,]?\\s*(\\d+)(\\((\\d+)\\))?:\\s*(\\d+)(\\s*-\\s*(\\d+))?(,\\s*(\\d{4})|\\s+\\((\\d{4})\\))?$" ),
+            m_jourRef5( "^([a-zA-Z. ]+)\\s+(vol\\.\\s+)?(\\d+),\\s+(\\d+)(\\([A-Z]+\\))?\\s+\\((\\d{4})\\)[.]?$" ),
+            m_jourRef6( "^([a-zA-Z. ]+),\\s+(\\d+)\\((\\d+)\\)\\s+(\\(([A-Za-z]+\\s+)?(\\d{4})\\))?\\s+(\\d+)(-(\\d+))?$" )
     {
         m_importer = new BibTeX::FileImporterBibTeX( FALSE );
         m_importer->setIgnoreComments( TRUE );
+        m_widget = new WebQueryArXivWidget( parent );
     }
 
     WebQueryArXiv::~WebQueryArXiv()
     {
+        delete m_widget;
         delete m_importer;
     }
 
@@ -51,7 +72,7 @@ namespace KBibTeX
 
     QString WebQueryArXiv::disclaimer()
     {
-        return i18n( "arXiv is an archive for electronic preprints" );
+        return i18n( "arXiv is an archive for preprints" );
     }
 
     QString WebQueryArXiv::disclaimerURL()
@@ -59,6 +80,11 @@ namespace KBibTeX
         return "http://www.arxiv.org/";
     }
 
+    WebQueryWidget *WebQueryArXiv::widget()
+    {
+        return m_widget;
+    }
+
     void WebQueryArXiv::cancelQuery()
     {
         m_urls.clear();
@@ -66,18 +92,21 @@ namespace KBibTeX
         // if ( m_currentJob != NULL ) m_currentJob->kill( FALSE );
     }
 
-    void WebQueryArXiv::query( const QString& searchTerm, int numberOfResults )
+    void WebQueryArXiv::query()
     {
-        m_urls.clear();
-        WebQuery::query( searchTerm, numberOfResults );
+        WebQuery::query();
+        Settings *settings = Settings::self();
+        settings->setWebQueryDefault( "ArXiv", m_widget->lineEditQuery->text() );
 
-        emit setTotalSteps( numberOfResults + 1 );
-        m_numberOfResults = numberOfResults;
+        m_urls.clear();
+        m_numberOfResults = m_widget->spinBoxMaxHits->value();
+        setNumStages( m_numberOfResults + 1 );
+
+        QString searchTerm = m_widget->lineEditQuery->text().stripWhiteSpace().replace( '$', "" );
         QStringList queryWords = QStringList::split( QRegExp( "\\s+" ), searchTerm );
-        if ( queryWords.size() == 0 )
+        if ( searchTerm.isEmpty() || queryWords.size() == 0 )
         {
-            emit setProgress( numberOfResults + 1 );
-            emit endSearch( false );
+            setEndSearch( WebQuery::statusInvalidQuery );
             return;
         }
 
@@ -86,7 +115,7 @@ namespace KBibTeX
             query = query.append( "AND " ).append( queryWords[i] ).append( " " );
         query.append( queryWords[queryWords.size()-1] );
 
-        KURL url = KURL( QString( "http://www.arxiv.org/find/all/1/all:+%2/0/1/0/all/0/1?per_page=%1" ).arg( numberOfResults ).arg( query.replace( "%", "%25" ).replace( " ", "+" ).replace( "?", "%3F" ).replace( "&", "%26" ) ) );
+        KURL url = KURL( QString( "http://www.arxiv.org/find/all/1/all:+%2/0/1/0/all/0/1?per_page=%1" ).arg( m_numberOfResults ).arg( query.replace( "%", "%25" ).replace( "+", "%2B" ).replace( " ", "%20" ).replace( "#", "%23" ).replace( "&", "%26" ).replace( "?", "%3F" ) ) );
         m_currentJob = KIO::storedGet( url, FALSE, FALSE );
         connect( m_currentJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( arXivResult( KIO::Job * ) ) );
     }
@@ -97,7 +126,7 @@ namespace KBibTeX
 
         if ( job->error() == 0 && !m_aborted )
         {
-            emit setProgress( 1 );
+            enterNextStage();
 
             QBuffer data;
             data.open( IO_WriteOnly );
@@ -124,7 +153,7 @@ namespace KBibTeX
             }
 
             if ( m_totalHits == 0 )
-                emit endSearch( false );
+                setEndSearch( WebQuery::statusSuccess );
             else if ( !m_urls.isEmpty() )
             {
                 KURL url = m_urls.first();
@@ -134,7 +163,7 @@ namespace KBibTeX
             }
         }
         else
-            emit endSearch( true );
+            setEndSearch( WebQuery::statusError );
     }
 
     void WebQueryArXiv::arXivAbstractResult( KIO::Job *job )
@@ -144,7 +173,7 @@ namespace KBibTeX
         if ( job->error() == 0 && !m_aborted )
         {
             ++m_receivedHits;
-            emit setProgress( m_receivedHits + 1 );
+            enterNextStage();
 
             QBuffer data;
             data.open( IO_WriteOnly );
@@ -304,34 +333,117 @@ namespace KBibTeX
             }
 
             /** find journal reference */
-            int fromPage = 0;
-            int toPage = 0;
             p = result.find( "<td class=\"tablecell jref\">", 0 );
             p2 = result.find( "</td>", p + 1 );
             if ( p >= 0 && p2 >= 0 )
             {
                 QString jref = result.mid( p + 27, p2 - p - 27 );
-                BibTeX::EntryField * field = new BibTeX::EntryField( BibTeX::EntryField::ftJournal );
-                entry->addField( field );
-                field->setValue( new BibTeX::Value( jref ) );
-                entry->setEntryType( BibTeX::Entry::etArticle );
+                jref.replace( "\n", " " );
+                qDebug( "jref =<%s>", jref.latin1() );
 
-                p = jref.find( "[0-9]+[-][0-9]+", 0 );
-                if ( p >= 0 )
+                QString jTitle = "";
+                QString jVol = "";
+                QString jNo = "";
+                QString jYear = "";
+                QString jPage1 = "";
+                QString jPage2 = "";
+
+                // m_jourRef1( "^([a-zA-Z. ]+[a-zA-Z.])\\s*(\\d+)\\s+\\((\\d{4})\\)\\s+([0-9A-Z]+)(-([0-9A-Z]+))?$" )
+                if ( m_jourRef1.search( jref ) == 0 )
                 {
-                    p2 = jref.find( "-", p );
-                    bool ok = FALSE;
-                    fromPage = jref.mid( p, p2 - p ).toInt( &ok );
-                    if ( !ok ) fromPage = 0;
-                    p = jref.find( QRegExp( "\\b|$" ), p2 + 1 );
-                    toPage = jref.mid( p2 + 1, p - p2 - 1 ).toInt( &ok );
-                    if ( !ok ) toPage = 0;
-                    if ( fromPage > 0 && toPage > 0 && ok )
-                    {
-                        BibTeX::EntryField * field  = new BibTeX::EntryField( BibTeX::EntryField::ftPages );
-                        entry->addField( field );
-                        field->setValue( new BibTeX::Value( QString( "%1--%2" ).arg( fromPage ).arg( toPage ) ) );
-                    }
+                    jTitle = m_jourRef1.cap( 1 );
+                    jVol = m_jourRef1.cap( 2 );
+                    jYear = m_jourRef1.cap( 3 );
+                    jPage1 = m_jourRef1.cap( 4 );
+                    jPage2 = m_jourRef1.cap( 6 );
+                }
+                // m_jourRef2( "^([a-zA-Z. ]+[a-zA-Z.]),\s+Vol[.]?\s+(\d+)[,]?\s+No[.]?\s+(\d+)\s+\((\d{4})\)[,]?\s+(pp\.\s+)?(\d+)(-(\d+))?$" )
+                else if ( m_jourRef2.search( jref ) == 0 )
+                {
+                    jTitle = m_jourRef2.cap( 1 );
+                    jVol = m_jourRef2.cap( 2 );
+                    jNo = m_jourRef2.cap( 3 );
+                    jYear = m_jourRef2.cap( 4 );
+                    jPage1 = m_jourRef2.cap( 6 );
+                    jPage2 = m_jourRef2.cap( 8 );
+                }
+                // m_jourRef3( "^([a-zA-Z. ]+),\\s+volume\\s+(\\d+),\\s+number\\s+(\\d+),\\s+pp\\.\\s+(\\d+)(-(\\d+))?,\\s+(\\d{4})$" )
+                else if ( m_jourRef3.search( jref ) == 0 )
+                {
+                    jTitle = m_jourRef3.cap( 1 );
+                    jVol = m_jourRef3.cap( 2 );
+                    jNo = m_jourRef3.cap( 3 );
+                    jPage1 = m_jourRef3.cap( 4 );
+                    jPage2 = m_jourRef3.cap( 6 );
+                    jYear = m_jourRef3.cap( 7 );
+                }
+                // m_jourRef4("^([a-zA-Z. ()]+[a-zA-Z.()])[,]?\\s*(\\d+)(\\((\\d+)\\))?:\\s*(\\d+)(\\s*-\\s*(\\d+))?(,\\s*(\\d{4})|\\s+\\((\\d{4})\\))?$")
+                else if ( m_jourRef4.search( jref ) == 0 )
+                {
+                    jTitle = m_jourRef4.cap( 1 );
+                    jVol = m_jourRef4.cap( 2 );
+                    jNo = m_jourRef4.cap( 4 );
+                    jPage1 = m_jourRef4.cap( 5 );
+                    jPage2 = m_jourRef4.cap( 7 );
+                    jYear = m_jourRef4.cap( 9 ).append( m_jourRef4.cap( 10 ) );
+                }
+                // m_jourRef5("^([a-zA-Z. ]+)\\s+(vol\\.\\s+)?(\\d+),\\s+(\\d+)(\\([A-Z]+\\))?\\s+\\((\\d{4})\\)[.]?$")
+                else if ( m_jourRef5.search( jref ) == 0 )
+                {
+                    jTitle = m_jourRef5.cap( 1 );
+                    jVol = m_jourRef5.cap( 3 );
+                    jPage1 = m_jourRef5.cap( 4 );
+                    jYear = m_jourRef5.cap( 6 );
+                }
+                // m_jourRef6("^([a-zA-Z. ]+),\\s+(\\d+)\\((\\d+)\\)\\s+(\\(([A-Za-z]+\\s+)?(\\d{4})\\))?\\s+(\\d+)(-(\\d+))?$")
+                else if ( m_jourRef6.search( jref ) == 0 )
+                {
+                    jTitle = m_jourRef6.cap( 1 );
+                    jVol = m_jourRef6.cap( 2 );
+                    jNo = m_jourRef6.cap( 3 );
+                    jYear = m_jourRef6.cap( 6 );
+                    jPage1 = m_jourRef6.cap( 7 );
+                    jPage2 = m_jourRef6.cap( 9 );
+                }
+
+                if ( !jTitle.isEmpty() )
+                {
+                    entry->deleteField( BibTeX::EntryField::ftJournal );
+                    BibTeX::EntryField * field = new BibTeX::EntryField( BibTeX::EntryField::ftJournal );
+                    entry->addField( field );
+                    field->setValue( new BibTeX::Value( jTitle ) );
+                    entry->setEntryType( BibTeX::Entry::etArticle );
+                }
+                if ( !jVol.isEmpty() )
+                {
+                    entry->deleteField( BibTeX::EntryField::ftVolume );
+                    BibTeX::EntryField * field = new BibTeX::EntryField( BibTeX::EntryField::ftVolume );
+                    entry->addField( field );
+                    field->setValue( new BibTeX::Value( jVol ) );
+                }
+                if ( !jNo.isEmpty() )
+                {
+                    entry->deleteField( BibTeX::EntryField::ftNumber );
+                    BibTeX::EntryField * field = new BibTeX::EntryField( BibTeX::EntryField::ftNumber );
+                    entry->addField( field );
+                    field->setValue( new BibTeX::Value( jNo ) );
+                }
+                if ( !jYear.isEmpty() )
+                {
+                    entry->deleteField( BibTeX::EntryField::ftYear );
+                    entry->deleteField( BibTeX::EntryField::ftMonth );
+                    BibTeX::EntryField * field = new BibTeX::EntryField( BibTeX::EntryField::ftYear );
+                    entry->addField( field );
+                    field->setValue( new BibTeX::Value( jYear ) );
+                }
+                if ( !jPage1.isEmpty() )
+                {
+                    entry->deleteField( BibTeX::EntryField::ftPages );
+                    BibTeX::EntryField * field = new BibTeX::EntryField( BibTeX::EntryField::ftPages );
+                    entry->addField( field );
+                    QString text = jPage1;
+                    if ( !jPage2.isEmpty() ) text.append( "--" ).append( jPage2 );
+                    field->setValue( new BibTeX::Value( text ) );
                 }
             }
 
@@ -358,10 +470,10 @@ namespace KBibTeX
             entry->addField( field );
             field->setValue( new BibTeX::Value( QString( "http://arxiv.org/pdf/%1" ).arg( id ) ) );
 
-            emit foundEntry( entry );
+            emit foundEntry( entry, false );
 
             if ( m_totalHits == m_receivedHits )
-                emit endSearch( false );
+                setEndSearch( WebQuery::statusSuccess );
             else if ( !m_urls.isEmpty() )
             {
                 KURL url = m_urls.first();
@@ -371,7 +483,7 @@ namespace KBibTeX
             }
         }
         else
-            emit endSearch( true );
+            setEndSearch( WebQuery::statusError );
     }
 
 }
