@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2005 by Thomas Fischer                             *
+ *   Copyright (C) 2004-2009 by Thomas Fischer                             *
  *   fischer@unix-ag.uni-kl.de                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,24 +20,43 @@
 #include <qfile.h>
 #include <qapplication.h>
 #include <qregexp.h>
+#include <qtimer.h>
 #include <qmap.h>
 #include <qbuffer.h>
+#include <qspinbox.h>
 
+#include <klineedit.h>
 #include <klocale.h>
+#include <kdebug.h>
 #include <kmessagebox.h>
 
+#include <settings.h>
 #include "webquerygooglescholar.h"
 
 namespace KBibTeX
 {
-    WebQueryGoogleScholar::WebQueryGoogleScholar( QWidget* parent ): WebQuery( parent ), m_http( NULL ), m_buffer( NULL )
+    WebQueryGoogleScholarWidget::WebQueryGoogleScholarWidget( QWidget *parent, const char *name )
+            : WebQueryWidget( parent, name )
+    {
+        init();
+
+        Settings *settings = Settings::self();
+        QString value = settings->getWebQueryDefault( "GoogleScholar" );
+        value = value == QString::null ? "" : value;
+        lineEditQuery->setText( value );
+        slotTextChanged( value, true );
+    }
+
+    WebQueryGoogleScholar::WebQueryGoogleScholar( QWidget* parent ): WebQuery( parent ), m_http( NULL ), m_restarting( false ), m_restartEnabled( false ), m_buffer( NULL )
     {
         m_importer = new BibTeX::FileImporterBibTeX( FALSE );
         m_importer->setIgnoreComments( TRUE );
+        m_widget = new WebQueryGoogleScholarWidget( parent );
     }
 
     WebQueryGoogleScholar::~WebQueryGoogleScholar()
     {
+        delete m_widget;
         delete m_importer;
         if ( m_buffer != NULL ) delete m_buffer;
         if ( m_http != NULL ) delete m_http;
@@ -58,71 +77,102 @@ namespace KBibTeX
         return "http://scholar.google.com/intl/en/scholar/about.html";
     }
 
-    void WebQueryGoogleScholar::query( const QString& searchTerm, int numberOfResults )
+    WebQueryWidget *WebQueryGoogleScholar::widget()
     {
-        WebQuery::query( searchTerm, numberOfResults );
+        return m_widget;
+    }
 
-        m_searchTerm = searchTerm;
-        m_numberOfResults = numberOfResults;
+    void WebQueryGoogleScholar::query()
+    {
+        WebQuery::query();
+        Settings *settings = Settings::self();
+        settings->setWebQueryDefault( "GoogleScholar", m_widget->lineEditQuery->text() );
+
+        m_searchTerm = m_widget->lineEditQuery->text().stripWhiteSpace().replace( '$', "" );
+        m_searchTerm = m_searchTerm.replace( "%", "%25" ).replace( "+", "%2B" ).replace( " ", "%20" ).replace( "#", "%23" ).replace( "&", "%26" ).replace( "?", "%3F" );
+
+        m_numberOfResults = m_widget->spinBoxMaxHits->value();
         m_currentRequestNumber = 0;
-        emit setTotalSteps( numberOfResults + 7 );
+        setNumStages( m_numberOfResults + 7 );
+
+        if ( m_searchTerm.isEmpty() )
+        {
+            setEndSearch( WebQuery::statusInvalidQuery );
+            return;
+        }
 
         m_http = new QHttp();
         connect( m_http, SIGNAL( done( bool ) ), this, SLOT( done( bool ) ) );
         connect( m_http, SIGNAL( responseHeaderReceived( const QHttpResponseHeader & ) ), this, SLOT( headerReceived( const QHttpResponseHeader & ) ) );
 
+        m_cookieMap.clear();
+        m_hits.clear();
+        m_redirectLocation = "";
         m_state = gsInit;
-        m_referer = QUrl( "http://www.google.com/" );
-        QUrl url( "http://scholar.google.com/" );
+        m_referer = KURL( "http://www.google.com/" );
+        KURL url( "http://scholar.google.com/" );
         startRequest( url );
+
+        m_restartEnabled = false;
+        QTimer::singleShot( 2000, this, SLOT( restart() ) );
     }
 
     void WebQueryGoogleScholar::cancelQuery()
     {
         if ( m_http != NULL )
         {
-            qDebug( "state= %d", m_http->state() );
+            kdDebug() << "state= " << m_http->state() << endl;
+            kdDebug() <<  "m_aborted= " << ( m_aborted ? "TRUE" : "FALSE" ) << endl;
             m_http->abort();
-            emit endSearch( true );
+            setEndSearch( WebQuery::statusError );
         }
     }
 
     void WebQueryGoogleScholar::done( bool error )
     {
+        kdDebug() << "WebQueryGoogleScholar::done ( " << error << ", " << m_restarting << " )" << endl;
+
         m_buffer->close();
 
+        if ( m_restarting )
+            return;
+
         if ( m_aborted )
-            emit endSearch( false );
+        {
+            setEndSearch( WebQuery::statusAborted );
+        }
         else if ( error )
         {
+            m_restartEnabled = false;
             KMessageBox::error( m_parent, QString( i18n( "Querying database '%1' failed." ) ).arg( title() ) );
-            emit endSearch( true );
+            setEndSearch( WebQuery::statusError );
         }
         else
         {
-            emit setProgress( ++m_currentRequestNumber );
+            ++m_currentRequestNumber;
+            enterNextStage();
 
             if ( !m_redirectLocation.isEmpty() )
             {
-                QUrl url( m_redirectLocation );
+                KURL url( m_redirectLocation );
                 startRequest( url );
             }
             else if ( m_state == gsInit &&  m_referer.host().contains( "scholar." ) )
             {
                 m_state = gsGetPref;
-                QUrl url = QUrl( QString( "http://%1/scholar_preferences" ).arg( m_referer.host() ) );
+                KURL url = KURL( QString( "http://%1/scholar_preferences" ).arg( m_referer.host() ) );
                 startRequest( url );
             }
             else if ( m_state == gsGetPref )
             {
                 m_state = gsSetPref;
-                QUrl url = QUrl( QString( "http://%1/scholar_setprefs?inststart=0&hl=en&lang=all&instq=&inst=cat-oclc&num=%2&scis=yes&scisf=4&submit=Save+Preferences" ).arg( m_referer.host() ).arg( m_numberOfResults ) );
+                KURL url = KURL( QString( "http://%1/scholar_setprefs?inststart=0&hl=en&lang=all&instq=&inst=cat-oclc&num=%2&scis=yes&scisf=4&submit=Save+Preferences" ).arg( m_referer.host() ).arg( m_numberOfResults ) );
                 startRequest( url );
             }
             else if ( m_state == gsSetPref )
             {
                 m_state = gsQuery;
-                QUrl url = QUrl( QString( "http://%1/scholar?q=%2&hl=en&lr=&num=%3&ie=UTF-8&btnG=Search&lr=" ).arg( m_referer.host() ).arg( m_searchTerm.replace( "%", "%25" ).replace( " ", "+" ).replace( "?", "%3F" ).replace( "&", "%26" ) ).arg( m_numberOfResults ) );
+                KURL url = KURL( QString( "http://%1/scholar?q=%3&hl=en&lr=&num=%2&ie=UTF-8&btnG=Search&lr=" ).arg( m_referer.host() ).arg( m_numberOfResults ).arg( m_searchTerm ) );
                 startRequest( url );
             }
             else if ( m_state == gsQuery )
@@ -147,41 +197,49 @@ namespace KBibTeX
                     QString hit = *m_hits.begin();
                     m_hits.remove( m_hits.begin() );
 
-                    QUrl url = QUrl( QString( "http://%1%2" ).arg( m_referer.host() ).arg( hit ) );
+                    KURL url = KURL( QString( "http://%1%2" ).arg( m_referer.host() ).arg( hit ) );
                     startRequest( url );
                 }
                 else
-                    emit endSearch( false );
+                    setEndSearch( WebQuery::statusSuccess );
             }
             else if ( m_state == gsResult )
             {
                 m_buffer->open( IO_ReadOnly );
                 BibTeX::File *tmpBibFile = m_importer->load( m_buffer );
-                for ( BibTeX::File::ElementList::iterator it = tmpBibFile->begin(); it != tmpBibFile->end(); ++it )
-                {
-                    BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( *it );
-                    if ( entry != NULL )
-                        emit foundEntry( new BibTeX::Entry( entry ) );
-                }
-                delete tmpBibFile;
                 m_buffer->close();
 
+                if ( tmpBibFile != NULL )
+                {
+                    for ( BibTeX::File::ElementList::iterator it = tmpBibFile->begin(); it != tmpBibFile->end(); ++it )
+                    {
+                        BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( *it );
+
+                        if ( entry != NULL )
+                            emit foundEntry( new BibTeX::Entry( entry ), false );
+                    }
+
+                    delete tmpBibFile;
+                }
+
                 if ( m_hits.size() > 0 )
                 {
                     QString hit = *m_hits.begin();
                     m_hits.remove( m_hits.begin() );
 
-                    QUrl url = QUrl( QString( "http://%1%2" ).arg( m_referer.host() ).arg( hit ) );
+                    KURL url = KURL( QString( "http://%1%2" ).arg( m_referer.host() ).arg( hit ) );
                     startRequest( url );
                 }
                 else
-                    emit endSearch( false );
+                    setEndSearch( WebQuery::statusSuccess );
             }
         }
     }
 
     void WebQueryGoogleScholar::headerReceived( const QHttpResponseHeader &resp )
     {
+        kdDebug() << "WebQueryGoogleScholar::headerReceived ( " << resp.toString() << " )" << endl;
+
         m_redirectLocation = "";
 
         QStringList keys = resp.keys();
@@ -201,14 +259,27 @@ namespace KBibTeX
 
     }
 
-    void WebQueryGoogleScholar::startRequest( QUrl& url )
+    void WebQueryGoogleScholar::restart()
     {
+        if ( m_state == gsInit && m_restartEnabled )
+        {
+            m_restarting = true;
+            qDebug( "Nothing happens, restarting search" );
+            m_http->abort();
+            m_restarting = false;
+            query();
+        }
+    }
+
+    void WebQueryGoogleScholar::startRequest( KURL& url )
+    {
+        kdDebug() << "WebQueryGoogleScholar::startRequest ( " << url.prettyURL() << " )" << endl;
         QHttpRequestHeader header( "GET", url.encodedPathAndQuery() );
         QString userAgent = "Mozilla/4.61 [dk] (OS/2; U)";
         header.setValue( "Host", url.host() );
         header.setValue( "User-Agent", userAgent );
         header.setValue( "Cookie", getCookies() );
-        header.setValue( "Referer", m_referer.toString() );
+        header.setValue( "Referer", m_referer.prettyURL() );
         m_referer = url;
 
         if ( m_buffer != NULL )
