@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2005 by Thomas Fischer                             *
+ *   Copyright (C) 2004-2009 by Thomas Fischer                             *
  *   fischer@unix-ag.uni-kl.de                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,10 +20,11 @@
 #ifndef KBIBTEXWEBQUERYGOOGLESCHOLAR_H
 #define KBIBTEXWEBQUERYGOOGLESCHOLAR_H
 
-#include <qurl.h>
 #include <qhttp.h>
 #include <qstringlist.h>
 
+#include <kurl.h>
+
 #include <fileimporterbibtex.h>
 #include <webquery.h>
 
@@ -31,6 +32,15 @@ class QBuffer;
 
 namespace KBibTeX
 {
+    /**
+     @author Thomas Fischer <fischer@unix-ag.uni-kl.de>
+    */
+    class WebQueryGoogleScholarWidget : public WebQueryWidget
+    {
+        Q_OBJECT
+    public:
+        WebQueryGoogleScholarWidget( QWidget *parent, const char *name = 0 );
+    };
 
     /**
      @author Thomas Fischer <fischer@unix-ag.uni-kl.de>
@@ -42,21 +52,26 @@ namespace KBibTeX
         WebQueryGoogleScholar( QWidget* parent );
         virtual ~WebQueryGoogleScholar();
 
-        void query( const QString& searchTerm, int numberOfResults );
-        void cancelQuery();
+        void query();
 
         QString title();
         QString disclaimer();
         QString disclaimerURL();
 
+        WebQueryWidget *widget();
+
+    protected:
+        void cancelQuery();
+
     protected slots:
         void done( bool );
         void headerReceived( const QHttpResponseHeader & );
+        void restart();
 
     private:
         enum GoogleState { gsInit, gsGetPref, gsSetPref, gsQuery, gsResult };
 
-        void startRequest( QUrl& url );
+        void startRequest( KURL& url );
         QString getCookies();
 
         int m_currentRequestNumber;
@@ -64,12 +79,15 @@ namespace KBibTeX
         int m_numberOfResults;
         GoogleState m_state;
         BibTeX::FileImporterBibTeX *m_importer;
-        QUrl m_referer;
+        KURL m_referer;
         QMap<QString, QString> m_cookieMap;
         QString m_redirectLocation;
         QHttp *m_http;
+        bool m_restarting;
+        bool m_restartEnabled;
         QBuffer *m_buffer;
         QStringList m_hits;
+        WebQueryGoogleScholarWidget *m_widget;
     };
 
 }
